'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fetch = require('cross-fetch');
var core = require('@urql/core');
var exchangeGraphcache = require('@urql/exchange-graphcache');
var tslib = require('tslib');
var zod = require('zod');
var graphql = require('graphql');
var viem = require('viem');
var chains = require('viem/chains');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n["default"] = e;
    return Object.freeze(n);
}

var fetch__default = /*#__PURE__*/_interopDefaultLegacy(fetch);
var viem__namespace = /*#__PURE__*/_interopNamespace(viem);

function isConnection(object) {
    return (typeof object === 'object' &&
        !!object &&
        ('edges' in object ||
            'total' in object ||
            'pageInfo' in object ||
            'breadcrumbs' in object));
}
function removeNodesAndEdges(data) {
    const keys = Object.keys(data);
    const output = {};
    keys.forEach((key) => {
        const value = data[key];
        if (typeof value === 'string' ||
            typeof value === 'boolean' ||
            typeof value === 'number' ||
            value === null ||
            value === undefined) {
            return (output[key] = value);
        }
        if (Array.isArray(value)) {
            // An array can just be an array of strings and not edges and nodes
            if (value.every((val) => typeof val === 'string')) {
                return (output[key] = value);
            }
            return (output[key] = value.map((item) => removeNodesAndEdges(item)));
        }
        if (isConnection(value)) {
            if (value.breadcrumbs)
                output[`${key}Breadcrumbs`] = value.breadcrumbs;
            if (value.total)
                output[`${key}Total`] = value.total;
            if (value.viewport)
                output[`${key}Viewport`] = value.viewport;
            if (value.pageInfo) {
                const { ...pageInfoRest } = value.pageInfo;
                output[`${key}PageInfo`] = pageInfoRest;
            }
            return (output[key] = value.edges?.map((item) => {
                if (item.node) {
                    return removeNodesAndEdges(item.node);
                }
                return item;
            }));
        }
        return (output[key] = removeNodesAndEdges(value));
    });
    return output;
}

class CustomUrqlClient {
    constructor(urqlClient) {
        this.urqlClient = urqlClient;
    }
    async query(options) {
        const { query, variables, ...additionalOptions } = options;
        const result = await this.urqlClient.query(query, variables, additionalOptions);
        const { error } = result;
        if (error) {
            console.error(error.stack);
            throw error;
        }
        return {
            ...result,
            data: result?.data &&
                removeNodesAndEdges(result.data),
        };
    }
}

// Below are zod enums used a validators that should be the same as the string unions generated by codegen.
// There are compile-time checks to ensure that they are the same and in sync. If you are getting a
// type error here, you need to update the the validator array to match the codegen type.
const CONTRACT_STANDARDS = ['ERC20', 'ERC721', 'ERC1155'];
const isContractStandard = zod.z.enum(CONTRACT_STANDARDS);
const MARKETPLACES = [
    'BLUR',
    'CRYPTOPUNKS',
    'LOOKSRARE',
    'NIFTY_GATEWAY',
    'OPENSEA',
    'SEAPORT',
    'X2Y2',
    'ZEROX',
];
const isMarketplace = zod.z.enum(MARKETPLACES);
const TOKEN_TRANSFER_TYPES = [
    'TRANSFER',
    'MINT',
    'SALE',
    'SWAP',
    'BURN',
];
const isTokenTransferType = zod.z.enum(TOKEN_TRANSFER_TYPES);

const supportedChains = [
    'ethereum',
    'polygon',
    'ethereumSepolia',
];

/**
 * https://eips.ethereum.org/EIPS/eip-137#name-syntax
 *
 * explanation of the regex pattern:
 * ^  Matches the start of the string
 * (?=.{3,255}$) Lookahead assertion for the length of the string (3 to 255 characters)
 * [\p{L}\p{N}\p{Pd}\p{M}\p{S}\u{1F300}-\u{1F6FF}]+  Matches one or more Unicode letter, number, punctuation, symbol, or character in the specified emoji range
 * (\.[\p{L}\p{N}\p{Pd}\p{M}\p{S}\u{1F300}-\u{1F6FF}]+)*  Matches zero or more occurrences of a dot followed by one or more Unicode characters
 * \.(?:eth|xyz|art)  Matches a dot followed by either 'eth', 'xyz', or 'art'
 * $  Matches the end of the string
 *
 */
function isValidENSAddress(ensAddress) {
    const allowedTLDs = ['eth', 'xyz', 'art'];
    const unicodeAndEmojis = '[\\p{L}\\p{N}\\p{Pd}\\p{M}\\p{S}\\u{1F300}-\\u{1F6FF}]';
    const regexPattern = `^(?=.{3,255}$)${unicodeAndEmojis}+(\\.${unicodeAndEmojis}+)*\\.(?:${allowedTLDs.join('|')})$`;
    const regex = new RegExp(regexPattern, 'mu');
    return regex.test(ensAddress);
}

const isEvmAddress = zod.z
    .string()
    .length(42) // Using built-in function for better error messages
    .startsWith('0x') // Using built-in function for better error messages
    .regex(/^0x[a-fA-F0-9]{40}$/, 'Not a valid address');
const isENSAddress = zod.z.string().refine((val) => isValidENSAddress(val));
const isEvmTransactionHash = zod.z
    .string()
    .length(66) // Using built-in function for better error messages
    .startsWith('0x') // Using built-in function for better error messages
    .regex(/^0x[a-fA-F0-9]{64}$/, 'Not a valid transaction hash');
const supportedChainInput = zod.z
    .object({
    chain: zod.z.enum(supportedChains).nullish(),
})
    .strict();
function fullFilters(baseType) {
    return zod.z
        .object({
        eq: baseType.nullish(),
        gt: baseType.nullish(),
        gte: baseType.nullish(),
        in: zod.z.array(baseType).nullish(),
        lt: baseType.nullish(),
        lte: baseType.nullish(),
        notIn: zod.z.array(baseType).nullish(),
    })
        .strict();
}
function limitedFilters(baseType) {
    return zod.z
        .object({
        eq: baseType.nullish(),
        in: zod.z.array(baseType).nullish(),
        notIn: zod.z.array(baseType).nullish(),
    })
        .strict();
}
const tokenEventFilters = zod.z
    .object({
    blockNumber: fullFilters(zod.z.number().positive()).nullish(),
    contractAddress: limitedFilters(isEvmAddress).nullish(),
    contractStandard: limitedFilters(isContractStandard).nullish(),
    fromAddress: limitedFilters(isEvmAddress).nullish(),
    marketplace: limitedFilters(isMarketplace).nullish(),
    timestamp: fullFilters(zod.z.string().datetime({ offset: true })).nullish(),
    toAddress: limitedFilters(isEvmAddress).nullish(),
    transactionHash: limitedFilters(isEvmTransactionHash).nullish(),
    type: limitedFilters(isTokenTransferType).nullish(),
    walletAddress: limitedFilters(isEvmAddress).nullish(),
})
    .strict();
const transactionFilters = zod.z
    .object({
    blockNumber: fullFilters(zod.z.number().positive()).nullish(),
    fromAddress: isEvmAddress.nullish(),
    timestamp: fullFilters(zod.z.string().datetime({ offset: true })).nullish(),
    toAddress: isEvmAddress.nullish(),
})
    .strict();
const gasPriceFilters = zod.z
    .object({
    blockNumber: fullFilters(zod.z.number().positive()).nullish(),
})
    .strict();
const paginationParams = zod.z
    .object({
    before: zod.z.string().nullish(),
    after: zod.z.string().nullish(),
    first: zod.z.number().positive().nullish(),
})
    .strict();
const baseEventsInput = zod.z
    .object({
    filter: tokenEventFilters.nullish(),
})
    .merge(paginationParams)
    .strict();
const baseTransactionsInput = zod.z
    .object({
    filter: transactionFilters.optional(),
})
    .merge(paginationParams)
    .strict();
const contractTokensFilter = zod.z
    .array(zod.z
    .object({
    contractAddress: isEvmAddress,
    tokenId: zod.z.string().optional(),
})
    .strict())
    .nonempty();
const rpcPaginationParams = zod.z
    .object({
    perPage: zod.z.number().positive().nullish(),
    page: zod.z.number().positive().nullish(),
})
    .strict();

const walletByAddressValidator = zod.z
    .object({
    address: zod.z.union([isENSAddress, isEvmAddress]),
    filter: zod.z
        .object({
        contractTokens: contractTokensFilter,
    })
        .strict()
        .optional(),
})
    .merge(paginationParams)
    .merge(supportedChainInput)
    .strict();

const nftDetailsValidator = zod.z
    .object({ contractAddress: isEvmAddress, tokenId: zod.z.string() })
    .merge(supportedChainInput)
    .strict();

const nftCollectionDetailsValidator = zod.z
    .object({
    contractAddress: isEvmAddress,
})
    .merge(supportedChainInput)
    .strict();

const nftTrendingCollectionsValidator = paginationParams
    .merge(supportedChainInput)
    .strict();

const nftsByContractAddressValidator = zod.z
    .object({ contractAddress: isEvmAddress })
    .merge(paginationParams)
    .merge(supportedChainInput)
    .strict();

const verifyOwnershipValidator = zod.z
    .object({
    address: zod.z.union([isENSAddress, isEvmAddress]),
    nfts: contractTokensFilter,
})
    .merge(supportedChainInput)
    .strict();

const CodegenEthMainnetContractDetailsDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetContractDetails" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "ContractDetails" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "ContractInfo" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "Contract" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "__typename" } }, { "kind": "Field", "name": { "kind": "Name", "value": "abi" } }, { "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "isVerified" } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "supportedErcInterfaces" } }, { "kind": "Field", "name": { "kind": "Name", "value": "symbol" } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenContract" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "decimals" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "ContractDetails" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "contract" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "contractAddress" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "ContractInfo" } }] } }] } }] };
const CodegenEthereumMainnetEventsGetAllDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthereumMainnetEventsGetAll" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenEventsFilterInput" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "EventsGetAll" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TokenEventInfo" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "__typename" } }, { "kind": "Field", "name": { "kind": "Name", "value": "blockNumber" } }, { "kind": "Field", "name": { "kind": "Name", "value": "fromAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "timestamp" } }, { "kind": "Field", "name": { "kind": "Name", "value": "toAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transactionHash" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transferIndex" } }, { "kind": "Field", "name": { "kind": "Name", "value": "type" } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenBurnEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenTransferEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenMintEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenSaleEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "marketplace" } }, { "kind": "Field", "name": { "kind": "Name", "value": "receivedTokenContractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "receivedTokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "sentTokenId" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "EventsGetAll" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenEvents" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TokenEventInfo" } }] } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }] } }] } }] };
const CodegenEthereumMainnetEventsByContractDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthereumMainnetEventsByContract" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenEventsFilterInput" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "EventsByContract" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TokenEventInfo" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "__typename" } }, { "kind": "Field", "name": { "kind": "Name", "value": "blockNumber" } }, { "kind": "Field", "name": { "kind": "Name", "value": "fromAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "timestamp" } }, { "kind": "Field", "name": { "kind": "Name", "value": "toAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transactionHash" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transferIndex" } }, { "kind": "Field", "name": { "kind": "Name", "value": "type" } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenBurnEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenTransferEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenMintEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenSaleEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "marketplace" } }, { "kind": "Field", "name": { "kind": "Name", "value": "receivedTokenContractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "receivedTokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "sentTokenId" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "EventsByContract" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "contract" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "contractAddress" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenEvents" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TokenEventInfo" } }] } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }] } }] } }] } }] };
const CodegenEthMainnetEventsByCollectionDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetEventsByCollection" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenEventsFilterInput" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "CollectionEventsFragment" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TokenEventInfo" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "__typename" } }, { "kind": "Field", "name": { "kind": "Name", "value": "blockNumber" } }, { "kind": "Field", "name": { "kind": "Name", "value": "fromAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "timestamp" } }, { "kind": "Field", "name": { "kind": "Name", "value": "toAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transactionHash" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transferIndex" } }, { "kind": "Field", "name": { "kind": "Name", "value": "type" } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenBurnEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenTransferEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenMintEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenSaleEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "marketplace" } }, { "kind": "Field", "name": { "kind": "Name", "value": "receivedTokenContractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "receivedTokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "sentTokenId" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "CollectionEventsFragment" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "collection" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "contractAddress" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenEvents" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TokenEventInfo" } }] } }] } }] } }] } }] } }] };
const CodegenEthereumMainnetEventsByNftDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthereumMainnetEventsByNft" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "tokenId" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenEventsFilterInput" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "NftEventsFragment" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TokenEventInfo" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "__typename" } }, { "kind": "Field", "name": { "kind": "Name", "value": "blockNumber" } }, { "kind": "Field", "name": { "kind": "Name", "value": "fromAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "timestamp" } }, { "kind": "Field", "name": { "kind": "Name", "value": "toAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transactionHash" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transferIndex" } }, { "kind": "Field", "name": { "kind": "Name", "value": "type" } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenBurnEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenTransferEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenMintEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "tokenQuantity" } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "TokenSaleEvent" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "marketplace" } }, { "kind": "Field", "name": { "kind": "Name", "value": "receivedTokenContractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "receivedTokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "sentTokenId" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "NftEventsFragment" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "nft" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "contractAddress" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "tokenId" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "tokenId" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenEvents" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TokenEventInfo" } }] } }] } }] } }] } }] } }] };
const CodegenEthMainnetWalletNFTsByContractAddressDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetWalletNFTsByContractAddress" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "NftsByContractAddressFragment" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "ERC1155NFTNode" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "ERC1155NFT" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "__typename" } }, { "kind": "Field", "name": { "kind": "Name", "value": "animationUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "collectionSlug" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "description" } }, { "kind": "Field", "name": { "kind": "Name", "value": "externalUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "metadata" } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "wallets" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ensName" } }] } }] } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "ERC721NFTNode" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "ERC721NFT" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "__typename" } }, { "kind": "Field", "name": { "kind": "Name", "value": "animationUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "attributes" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "value" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "collectionSlug" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "description" } }, { "kind": "Field", "name": { "kind": "Name", "value": "externalUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "metadata" } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "Field", "name": { "kind": "Name", "value": "wallet" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ensName" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "NftsByContractAddressFragment" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "collection" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "contractAddress" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "__typename" } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "ERC1155Collection" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "nfts" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "ERC1155NFTNode" } }] } }] } }] } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "ERC721Collection" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "nfts" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "ERC721NFTNode" } }] } }] } }] } }] } }] } }] } }] };
const CodegenEthMainnetWalletNFTsByAddressDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetWalletNFTsByAddress" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "address" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "WalletNFTsFilterInput" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "WalletByAddressFragment" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "WalletNFTNode" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "WalletNFT" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "nft" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "animationUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "collectionSlug" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "description" } }, { "kind": "Field", "name": { "kind": "Name", "value": "externalUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "metadata" } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "WalletByAddressFragment" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "walletByAddress" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "address" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "address" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ensName" } }, { "kind": "Field", "name": { "kind": "Name", "value": "walletNFTs" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "WalletNFTNode" } }] } }] } }] } }] } }] } }] };
const CodegenEthMainnetWalletNFTsByEnsDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetWalletNFTsByEns" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "ensName" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "WalletNFTsFilterInput" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "WalletByEnsFragment" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "WalletNFTNode" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "WalletNFT" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "nft" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "animationUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "collectionSlug" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "description" } }, { "kind": "Field", "name": { "kind": "Name", "value": "externalUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "metadata" } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "WalletByEnsFragment" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "walletByENS" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "ensName" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "ensName" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ensName" } }, { "kind": "Field", "name": { "kind": "Name", "value": "walletNFTs" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "WalletNFTNode" } }] } }] } }] } }] } }] } }] };
const CodegenEthMainnetNFTDetailsDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetNFTDetails" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "tokenId" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "NftDetails" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "NftDetails" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "nft" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "contractAddress" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "tokenId" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "tokenId" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "__typename" } }, { "kind": "Field", "name": { "kind": "Name", "value": "animationUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "collectionSlug" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "description" } }, { "kind": "Field", "name": { "kind": "Name", "value": "externalUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "metadata" } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "ERC1155NFT" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "wallets" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ensName" } }] } }] } }] } }] } }, { "kind": "InlineFragment", "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "ERC721NFT" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "wallet" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ensName" } }] } }] } }] } }] } }] };
const CodegenEthMainnetNftCollectionDetailsDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetNftCollectionDetails" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "NftCollectionInfo" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "NftCollectionInfo" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "collection" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "contractAddress" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "contractAddress" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "bannerImage" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "height" } }, { "kind": "Field", "name": { "kind": "Name", "value": "mimeType" } }, { "kind": "Field", "name": { "kind": "Name", "value": "url" } }, { "kind": "Field", "name": { "kind": "Name", "value": "width" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "baseTokenUri" } }, { "kind": "Field", "name": { "kind": "Name", "value": "circulatingSupply" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contract" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "isVerified" } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "symbol" } }, { "kind": "Field", "name": { "kind": "Name", "value": "supportedErcInterfaces" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "description" } }, { "kind": "Field", "name": { "kind": "Name", "value": "externalUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "image" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "height" } }, { "kind": "Field", "name": { "kind": "Name", "value": "mimeType" } }, { "kind": "Field", "name": { "kind": "Name", "value": "url" } }, { "kind": "Field", "name": { "kind": "Name", "value": "width" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ohlcvChart" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "ObjectValue", "fields": [{ "kind": "ObjectField", "name": { "kind": "Name", "value": "limit" }, "value": { "kind": "IntValue", "value": "1" } }] } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "average" } }, { "kind": "Field", "name": { "kind": "Name", "value": "close" } }, { "kind": "Field", "name": { "kind": "Name", "value": "count" } }, { "kind": "Field", "name": { "kind": "Name", "value": "high" } }, { "kind": "Field", "name": { "kind": "Name", "value": "low" } }, { "kind": "Field", "name": { "kind": "Name", "value": "open" } }, { "kind": "Field", "name": { "kind": "Name", "value": "volume" } }, { "kind": "Field", "name": { "kind": "Name", "value": "timestamp" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "openseaMetadata" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "isHidden" } }, { "kind": "Field", "name": { "kind": "Name", "value": "isVerified" } }, { "kind": "Field", "name": { "kind": "Name", "value": "unsafeSlug" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "slug" } }, { "kind": "Field", "name": { "kind": "Name", "value": "symbol" } }, { "kind": "Field", "name": { "kind": "Name", "value": "totalSupply" } }, { "kind": "Field", "name": { "kind": "Name", "value": "twitterUsername" } }] } }] } }] };
const CodegenEthMainnetTrendingCollectionsDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetTrendingCollections" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "NftTrendingCollections" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TrendingCollectionInfo" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "Collection" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "baseTokenUri" } }, { "kind": "Field", "name": { "kind": "Name", "value": "circulatingSupply" } }, { "kind": "Field", "name": { "kind": "Name", "value": "description" } }, { "kind": "Field", "name": { "kind": "Name", "value": "externalUrl" } }, { "kind": "Field", "name": { "kind": "Name", "value": "image" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "height" } }, { "kind": "Field", "name": { "kind": "Name", "value": "mimeType" } }, { "kind": "Field", "name": { "kind": "Name", "value": "url" } }, { "kind": "Field", "name": { "kind": "Name", "value": "width" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "openseaMetadata" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "isHidden" } }, { "kind": "Field", "name": { "kind": "Name", "value": "isVerified" } }, { "kind": "Field", "name": { "kind": "Name", "value": "unsafeSlug" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "symbol" } }, { "kind": "Field", "name": { "kind": "Name", "value": "totalSupply" } }, { "kind": "Field", "name": { "kind": "Name", "value": "twitterUsername" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "NftTrendingCollections" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "trendingCollections" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "collection" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TrendingCollectionInfo" } }] } }] } }] } }] } }] } }] };
const CodegenEthMainnetVerifyOwnershipByAddressDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetVerifyOwnershipByAddress" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "address" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "WalletNFTsFilterInput" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "VerifyOwnershipInfo" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "VerifyOwnershipNFTDetails" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "WalletNFT" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "nft" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "VerifyOwnershipInfo" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "walletByAddress" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "address" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "address" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "walletNFTs" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "VerifyOwnershipNFTDetails" } }] } }] } }] } }] } }] } }] };
const CodegenEthMainnetVerifyOwnershipByENSDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetVerifyOwnershipByENS" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "ensName" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "WalletNFTsFilterInput" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "VerifyOwnershipInfoByENS" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "VerifyOwnershipNFTDetails" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "WalletNFT" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "nft" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenId" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "VerifyOwnershipInfoByENS" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "walletByENS" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "ensName" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "ensName" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "walletNFTs" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "VerifyOwnershipNFTDetails" } }] } }] } }] } }] } }] } }] };
const CodegenEthMainnetBalancesByWalletAddressDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetBalancesByWalletAddress" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "address" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "GetBalancesByWalletAddressFragment" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TokenBalanceNode" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "WalletTokenBalance" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "totalBalance" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contract" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "decimals" } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "symbol" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "GetBalancesByWalletAddressFragment" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "walletByAddress" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "address" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "address" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ensName" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenBalances" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TokenBalanceNode" } }] } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }] } }] } }] } }] };
const CodegenEthMainnetBalancesByWalletENSDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetBalancesByWalletENS" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "ensName" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "GetBalancesByWalletENSFragment" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TokenBalanceNode" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "WalletTokenBalance" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "totalBalance" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contract" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "decimals" } }, { "kind": "Field", "name": { "kind": "Name", "value": "name" } }, { "kind": "Field", "name": { "kind": "Name", "value": "symbol" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "GetBalancesByWalletENSFragment" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "walletByENS" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "ensName" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "ensName" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ensName" } }, { "kind": "Field", "name": { "kind": "Name", "value": "tokenBalances" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TokenBalanceNode" } }] } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }] } }] } }] } }] };
const CodegenEthMainnetTransactionsByHashDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetTransactionsByHash" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "hash" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TransactionsByHash" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TransactionsNode" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "Transaction" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "blockNumber" } }, { "kind": "Field", "name": { "kind": "Name", "value": "blockTimestamp" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "fromAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "cumulativeGasUsed" } }, { "kind": "Field", "name": { "kind": "Name", "value": "effectiveGasPrice" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gasPrice" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gasUsed" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hash" } }, { "kind": "Field", "name": { "kind": "Name", "value": "maxFeePerGas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "maxPriorityFeePerGas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "toAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "type" } }, { "kind": "Field", "name": { "kind": "Name", "value": "input" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transactionIndex" } }, { "kind": "Field", "name": { "kind": "Name", "value": "value" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TransactionsByHash" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "transaction" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "hash" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "hash" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TransactionsNode" } }] } }] } }] };
const CodegenEthMainnetTransactionsBySearchDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetTransactionsBySearch" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "TransactionsFilterInput" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TransactionsBySearch" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TransactionsNode" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "Transaction" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "blockNumber" } }, { "kind": "Field", "name": { "kind": "Name", "value": "blockTimestamp" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "fromAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "cumulativeGasUsed" } }, { "kind": "Field", "name": { "kind": "Name", "value": "effectiveGasPrice" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gasPrice" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gasUsed" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hash" } }, { "kind": "Field", "name": { "kind": "Name", "value": "maxFeePerGas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "maxPriorityFeePerGas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "toAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "type" } }, { "kind": "Field", "name": { "kind": "Name", "value": "input" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transactionIndex" } }, { "kind": "Field", "name": { "kind": "Name", "value": "value" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TransactionsBySearch" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "transactions" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TransactionsNode" } }] } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }] } }] } }] };
const CodegenEthMainnetTransactionsByWalletAddressDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetTransactionsByWalletAddress" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "address" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TransactionsByWalletAddress" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TransactionsNode" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "Transaction" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "blockNumber" } }, { "kind": "Field", "name": { "kind": "Name", "value": "blockTimestamp" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "fromAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "cumulativeGasUsed" } }, { "kind": "Field", "name": { "kind": "Name", "value": "effectiveGasPrice" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gasPrice" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gasUsed" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hash" } }, { "kind": "Field", "name": { "kind": "Name", "value": "maxFeePerGas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "maxPriorityFeePerGas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "toAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "type" } }, { "kind": "Field", "name": { "kind": "Name", "value": "input" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transactionIndex" } }, { "kind": "Field", "name": { "kind": "Name", "value": "value" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TransactionsByWalletAddress" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "walletByAddress" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "address" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "address" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ensName" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transactions" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TransactionsNode" } }] } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }] } }] } }] } }] };
const CodegenEthMainnetTransactionsByWalletENSDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetTransactionsByWalletENS" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "ensName" } }, "type": { "kind": "NonNullType", "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "Int" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }, { "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "String" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TransactionsByWalletENS" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TransactionsNode" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "Transaction" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "blockNumber" } }, { "kind": "Field", "name": { "kind": "Name", "value": "blockTimestamp" } }, { "kind": "Field", "name": { "kind": "Name", "value": "contractAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "fromAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "cumulativeGasUsed" } }, { "kind": "Field", "name": { "kind": "Name", "value": "effectiveGasPrice" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gasPrice" } }, { "kind": "Field", "name": { "kind": "Name", "value": "gasUsed" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hash" } }, { "kind": "Field", "name": { "kind": "Name", "value": "maxFeePerGas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "maxPriorityFeePerGas" } }, { "kind": "Field", "name": { "kind": "Name", "value": "toAddress" } }, { "kind": "Field", "name": { "kind": "Name", "value": "type" } }, { "kind": "Field", "name": { "kind": "Name", "value": "input" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transactionIndex" } }, { "kind": "Field", "name": { "kind": "Name", "value": "value" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "Pagination" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "PageInfo" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "endCursor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasNextPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "hasPreviousPage" } }, { "kind": "Field", "name": { "kind": "Name", "value": "startCursor" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "TransactionsByWalletENS" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "walletByENS" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "ensName" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "ensName" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "address" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ensName" } }, { "kind": "Field", "name": { "kind": "Name", "value": "transactions" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "first" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "first" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "before" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "before" } } }, { "kind": "Argument", "name": { "kind": "Name", "value": "after" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "after" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "edges" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "node" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "TransactionsNode" } }] } }] } }, { "kind": "Field", "name": { "kind": "Name", "value": "pageInfo" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "Pagination" } }] } }] } }] } }] } }] };
const CodegenEthMainnetGasPricesDocument = { "kind": "Document", "definitions": [{ "kind": "OperationDefinition", "operation": "query", "name": { "kind": "Name", "value": "EthMainnetGasPrices" }, "variableDefinitions": [{ "kind": "VariableDefinition", "variable": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } }, "type": { "kind": "NamedType", "name": { "kind": "Name", "value": "GasPriceFilterInput" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "ethereum" }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "GasPrice" } }] } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "GasPriceInfo" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "GasPrice" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "blockNumber" } }, { "kind": "Field", "name": { "kind": "Name", "value": "total" } }, { "kind": "Field", "name": { "kind": "Name", "value": "average" } }, { "kind": "Field", "name": { "kind": "Name", "value": "ceiling" } }, { "kind": "Field", "name": { "kind": "Name", "value": "floor" } }, { "kind": "Field", "name": { "kind": "Name", "value": "median" } }] } }, { "kind": "FragmentDefinition", "name": { "kind": "Name", "value": "GasPrice" }, "typeCondition": { "kind": "NamedType", "name": { "kind": "Name", "value": "EVMSchemaType" } }, "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "Field", "name": { "kind": "Name", "value": "gasPrices" }, "arguments": [{ "kind": "Argument", "name": { "kind": "Name", "value": "filter" }, "value": { "kind": "Variable", "name": { "kind": "Name", "value": "filter" } } }], "selectionSet": { "kind": "SelectionSet", "selections": [{ "kind": "FragmentSpread", "name": { "kind": "Name", "value": "GasPriceInfo" } }] } }] } }] };

// Formats the query result with the edges and nodes removed into what we want to return to the user
function formatQueryResult(queryResult, resultsKey, // Key that the actual results are in
paginationKey, // Key that the pagination info is in
resultsKeyToRemove, // Key that the results are nested under to remove for each entry
additionalModification) {
    const additionaProperties = Object.fromEntries(Object.entries(queryResult).filter(([key]) => key !== resultsKey && key !== paginationKey));
    let trimmedResults = queryResult[resultsKey];
    if (resultsKeyToRemove) {
        trimmedResults = trimmedResults.map((result) => result[resultsKeyToRemove] || {});
    }
    const formattedResultBase = {
        results: trimmedResults,
        pageInfo: queryResult[paginationKey],
    };
    let result = {
        ...formattedResultBase,
        ...additionaProperties,
    };
    if (additionalModification)
        result = additionalModification(result);
    return result;
}

const emptyPageInfo = {
    endCursor: null,
    hasNextPage: false,
    hasPreviousPage: false,
    startCursor: null,
};
function weiToGwei(wei) {
    return +(wei / 1e9).toFixed(2);
}

const DEFAULT_CHAIN = 'ethereum';

class QNInputValidationError extends Error {
    constructor({ messages, zodError, }) {
        super(`QuickNode SDK Input Validation Error: ${messages.join(', ')}`);
        this.messages = messages;
        this.issues = zodError.issues;
        this.zodError = zodError; // see https://github.com/colinhacks/zod/blob/HEAD/ERROR_HANDLING.md
    }
}

class QNInvalidEndpointUrl extends Error {
    constructor() {
        super('Endpoint URL is not in a valid QuickNode URL format. Please check the URL and try again');
    }
}

class QNChainNotSupported extends Error {
    constructor(endpointUrl) {
        super(`The chain for endpoint URL ${endpointUrl} is not currently supported by the QuickNode SDK.`);
    }
}

function formatErrors(baseError) {
    const errorMessages = [];
    baseError.errors.forEach((error) => {
        errorMessages.push(`${error.path.length > 0 ? error.path + ': ' : ''}${error.message}`);
    });
    return errorMessages.length > 0
        ? new QNInputValidationError({
            messages: errorMessages,
            zodError: baseError,
        })
        : null;
}
// Decorator for runtime validation to handle input that is unknown at compile time
function ValidateInput(schema) {
    return function (target, propertyName, descriptor) {
        const method = descriptor.value;
        if (!method)
            return;
        descriptor.value = async function (...args) {
            const [input] = args;
            const validation = schema.safeParse(input);
            if (!validation.success) {
                const formattedErrors = formatErrors(validation.error);
                if (formattedErrors) {
                    throw formattedErrors;
                }
            }
            return method.apply(this, args);
        };
    };
}

// Takes the generated query document and modifies the chain name to the one passed in
function modifyQueryForChain(chainName, documentNode) {
    documentNode.definitions = documentNode.definitions.map((doc) => {
        if (doc.kind === graphql.Kind.OPERATION_DEFINITION) {
            doc.selectionSet.selections = doc.selectionSet.selections.map((selection) => {
                if (selection.kind === graphql.Kind.FIELD &&
                    selection.name.kind == graphql.Kind.NAME &&
                    selection.name.value === 'ethereum') {
                    const updatedChainSelection = {
                        ...selection,
                        ...{
                            name: { ...selection.name, value: chainName },
                        },
                    };
                    return updatedChainSelection;
                }
                return selection;
            });
        }
        return doc;
    });
    return documentNode;
}

class NftsController {
    constructor(client, defaultChain = DEFAULT_CHAIN) {
        this.client = client;
        this.defaultChain = defaultChain;
    }
    async getByWallet(variables) {
        const { address, ...allVariables } = variables;
        if (isValidENSAddress(address)) {
            return this.getByWalletENS({
                ensName: address,
                ...allVariables,
            });
        }
        return this.getByWalletAddress({
            address,
            ...allVariables,
        });
    }
    async getByWalletENS(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetWalletNFTsByEnsDocument);
        const result = await this.client.query({
            query: query,
            variables: queryVariables,
        });
        const walletByENS = result?.data?.[userChain]?.walletByENS;
        if (!walletByENS?.walletNFTs?.length) {
            // Address can still be valid ENS name, but not have any NFTs
            const address = walletByENS?.address || '';
            const ensName = walletByENS?.ensName || '';
            return {
                address: address,
                ensName: ensName,
                results: [],
                pageInfo: emptyPageInfo,
            };
        }
        const formattedResult = formatQueryResult(walletByENS, 'walletNFTs', 'walletNFTsPageInfo', 'nft');
        return formattedResult;
    }
    async getByWalletAddress(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetWalletNFTsByAddressDocument);
        const result = await this.client.query({
            query: query,
            variables: queryVariables,
        });
        const walletByAddress = result?.data?.[userChain]?.walletByAddress;
        if (!walletByAddress?.walletNFTs?.length) {
            const address = walletByAddress?.address || '';
            const ensName = walletByAddress?.ensName || '';
            return {
                address: address,
                ensName: ensName,
                results: [],
                pageInfo: emptyPageInfo,
            };
        }
        const formattedResult = formatQueryResult(walletByAddress, 'walletNFTs', 'walletNFTsPageInfo', 'nft');
        return formattedResult;
    }
    async getTrendingCollections(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetTrendingCollectionsDocument);
        const result = await this.client.query({
            query: query,
            variables: queryVariables,
        });
        const chainResult = result?.data?.[userChain];
        if (chainResult?.trendingCollections?.length) {
            const formattedResult = formatQueryResult(chainResult, 'trendingCollections', 'trendingCollectionsPageInfo', 'collection');
            return formattedResult;
        }
        return {
            results: [],
            pageInfo: emptyPageInfo,
        };
    }
    async getByContractAddress(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetWalletNFTsByContractAddressDocument);
        const result = await this.client.query({
            query: query,
            variables: queryVariables,
        });
        const collection = result?.data?.[userChain]?.collection;
        if (!collection?.nfts?.length) {
            return {
                standard: null,
                results: [],
                pageInfo: emptyPageInfo,
            };
        }
        const setErcStandard = (results) => {
            const standardMap = {
                ERC1155Collection: 'ERC1155',
                ERC721Collection: 'ERC721',
            };
            // Remove address too since it was only used as a key field
            const { __typename, address, ...newResults } = results;
            return {
                ...newResults,
                standard: standardMap[results['__typename']] || null,
            };
        };
        const formattedResult = formatQueryResult(collection, 'nfts', 'nftsPageInfo', null, setErcStandard);
        return formattedResult;
    }
    async getNFTDetails(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetNFTDetailsDocument);
        const result = await this.client.query({
            query,
            variables: queryVariables,
        });
        const nft = result?.data?.[userChain]?.nft;
        if (nft)
            return { nft };
        return { nft: null };
    }
    async getCollectionDetails(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetNftCollectionDetailsDocument);
        const result = await this.client.query({
            query: query,
            variables: queryVariables,
        });
        const collection = result?.data?.[userChain]?.collection;
        if (collection)
            return { collection };
        return { collection: null };
    }
    async verifyOwnership(variables) {
        const { address, ...allVariables } = variables;
        if (isValidENSAddress(address)) {
            return this.verifyOwnershipByENS({
                ensName: address,
                ...allVariables,
            });
        }
        return this.verifyOwnershipByAddress({
            address,
            ...allVariables,
        });
    }
    async verifyOwnershipByAddress(variables) {
        const { chain, address, nfts } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetVerifyOwnershipByAddressDocument);
        const result = await this.client.query({
            query: query,
            variables: { address, filter: { contractTokens: nfts } },
        });
        const walletNFTs = result?.data?.[userChain]?.walletByAddress?.walletNFTs;
        return !!walletNFTs?.length;
    }
    async verifyOwnershipByENS(variables) {
        const { chain, ensName, nfts } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetVerifyOwnershipByENSDocument);
        const result = await this.client.query({
            query: query,
            variables: { ensName, filter: { contractTokens: nfts } },
        });
        const walletByENS = result?.data?.[userChain]?.walletByENS?.walletNFTs;
        return !!walletByENS?.length;
    }
}
tslib.__decorate([
    ValidateInput(walletByAddressValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], NftsController.prototype, "getByWallet", null);
tslib.__decorate([
    ValidateInput(nftTrendingCollectionsValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], NftsController.prototype, "getTrendingCollections", null);
tslib.__decorate([
    ValidateInput(nftsByContractAddressValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], NftsController.prototype, "getByContractAddress", null);
tslib.__decorate([
    ValidateInput(nftDetailsValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], NftsController.prototype, "getNFTDetails", null);
tslib.__decorate([
    ValidateInput(nftCollectionDetailsValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], NftsController.prototype, "getCollectionDetails", null);
tslib.__decorate([
    ValidateInput(verifyOwnershipValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], NftsController.prototype, "verifyOwnership", null);

const balancesByWalletAddressValidator = zod.z
    .object({
    address: zod.z.union([isENSAddress, isEvmAddress]),
})
    .merge(paginationParams)
    .merge(supportedChainInput)
    .strict();

class TokensController {
    constructor(client, defaultChain = DEFAULT_CHAIN) {
        this.client = client;
        this.defaultChain = defaultChain;
    }
    async getBalancesByWallet(variables) {
        const { address, ...allVariables } = variables;
        if (isValidENSAddress(address)) {
            return this.getBalancesByWalletENS({
                ensName: address,
                ...allVariables,
            });
        }
        return this.getBalancesByWalletAddress({
            address,
            ...allVariables,
        });
    }
    async getBalancesByWalletENS(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetBalancesByWalletENSDocument);
        const result = await this.client.query({
            variables: queryVariables,
            query: query,
        });
        const walletByENS = result?.data?.[userChain]?.walletByENS;
        if (!walletByENS?.tokenBalances?.length) {
            // Address can still be valid ENS name, but not have any balances
            const address = walletByENS?.address || '';
            const ensName = walletByENS?.ensName || '';
            return {
                address: address,
                ensName: ensName,
                results: [],
                pageInfo: emptyPageInfo,
            };
        }
        const formattedResult = formatQueryResult(walletByENS, 'tokenBalances', 'tokenBalancesPageInfo', null, this.flattenBalanceResponses // Remove the "contract" key and move info to balance result body
        );
        return formattedResult;
    }
    async getBalancesByWalletAddress(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetBalancesByWalletAddressDocument);
        const result = await this.client.query({
            variables: queryVariables,
            query: query,
        });
        const walletByAddress = result?.data?.[userChain]?.walletByAddress;
        if (!walletByAddress?.tokenBalances?.length) {
            // Address can still be valid address, but not have any balances
            const address = walletByAddress?.address || '';
            const ensName = walletByAddress?.ensName || '';
            return {
                address: address,
                ensName: ensName,
                results: [],
                pageInfo: emptyPageInfo,
            };
        }
        const formattedResult = formatQueryResult(walletByAddress, 'tokenBalances', 'tokenBalancesPageInfo', null, this.flattenBalanceResponses // Remove the "contract" key and move info to balance result body
        );
        return formattedResult;
    }
    flattenBalanceResponses(response) {
        const modifiedResults = response.results.map((result) => {
            const { contract: { ...contractInfo }, ...balanceInfo } = result;
            return { ...balanceInfo, ...contractInfo };
        });
        response.results = modifiedResults;
        return response;
    }
}
tslib.__decorate([
    ValidateInput(balancesByWalletAddressValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], TokensController.prototype, "getBalancesByWallet", null);

const gasPricesValidator = zod.z
    .object({
    returnInGwei: zod.z.boolean().nullish(),
    filter: gasPriceFilters.nullish(),
})
    .merge(supportedChainInput)
    .strict();

class UtilsController {
    constructor(client, defaultChain = DEFAULT_CHAIN) {
        this.client = client;
        this.defaultChain = defaultChain;
    }
    async getGasPrices(variables) {
        const { chain, ...queryVariables } = variables;
        const returnInGwei = variables.returnInGwei || false;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetGasPricesDocument);
        const result = await this.client.query({
            query: query,
            variables: queryVariables,
        });
        const gasPrices = result?.data?.[userChain]?.gasPrices;
        if (Array.isArray(gasPrices) && gasPrices.length > 0) {
            if (returnInGwei) {
                const fieldsToTransform = [
                    'total',
                    'average',
                    'ceiling',
                    'floor',
                    'median',
                ];
                const modifiedGasPrices = gasPrices.map((gasPrice) => {
                    fieldsToTransform.map((field) => {
                        gasPrice[field] = weiToGwei(gasPrice[field]);
                    });
                    return gasPrice;
                });
                return { gasPrices: modifiedGasPrices };
            }
            return { gasPrices };
        }
        return { gasPrices: [] };
    }
}
tslib.__decorate([
    ValidateInput(gasPricesValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], UtilsController.prototype, "getGasPrices", null);

// Using zod for runtime validation
const contractDetailsValidator = zod.z
    .object({
    contractAddress: isEvmAddress,
})
    .merge(supportedChainInput)
    .strict();

class ContractsController {
    constructor(client, defaultChain = DEFAULT_CHAIN) {
        this.client = client;
        this.defaultChain = defaultChain;
    }
    async getDetails(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetContractDetailsDocument);
        const result = await this.client.query({
            variables: queryVariables,
            query: query,
        });
        const contract = result?.data?.[userChain]?.contract;
        if (contract)
            return { contract };
        return { contract: null };
    }
}
tslib.__decorate([
    ValidateInput(contractDetailsValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], ContractsController.prototype, "getDetails", null);

const contractEventsValidator = zod.z
    .object({
    contractAddress: isEvmAddress,
})
    .merge(baseEventsInput)
    .merge(supportedChainInput)
    .strict();

const collectionEventsValidator = zod.z
    .object({
    contractAddress: isEvmAddress,
})
    .merge(baseEventsInput)
    .merge(supportedChainInput)
    .strict();

const nftEventsValidator = zod.z
    .object({
    contractAddress: isEvmAddress,
    tokenId: zod.z.string(),
})
    .merge(baseEventsInput)
    .merge(supportedChainInput)
    .strict();

const allEventsValidator = baseEventsInput
    .merge(supportedChainInput)
    .strict();

class EventsController {
    constructor(client, defaultChain = DEFAULT_CHAIN) {
        this.client = client;
        this.defaultChain = defaultChain;
    }
    async getByContract(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthereumMainnetEventsByContractDocument);
        const result = await this.client.query({
            query: query,
            variables: queryVariables,
        });
        const contract = result?.data?.[userChain]?.contract;
        if (!contract?.tokenEvents?.length) {
            return {
                results: [],
                pageInfo: emptyPageInfo,
            };
        }
        const formattedResult = formatQueryResult(contract, 'tokenEvents', 'tokenEventsPageInfo');
        return formattedResult;
    }
    async getByNFTCollection(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetEventsByCollectionDocument);
        const result = await this.client.query({
            query: query,
            variables: queryVariables,
        });
        const collection = result?.data?.[userChain]?.collection;
        if (!collection?.tokenEvents?.length)
            return { results: [], pageInfo: emptyPageInfo };
        function removeKeyFields(results) {
            const { address, ...newResults } = results;
            return newResults;
        }
        const formattedResult = formatQueryResult(collection, 'tokenEvents', 'tokenEventsPageInfo', null, removeKeyFields);
        return formattedResult;
    }
    async getByNFT(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthereumMainnetEventsByNftDocument);
        const result = await this.client.query({
            query: query,
            variables: queryVariables,
        });
        const nft = result?.data?.[userChain]?.nft;
        if (!nft?.tokenEvents?.length)
            return { results: [], pageInfo: emptyPageInfo };
        function removeKeyFields(results) {
            const { contractAddress, tokenId, ...newResults } = results;
            return newResults;
        }
        const formattedResult = formatQueryResult(nft, 'tokenEvents', 'tokenEventsPageInfo', null, removeKeyFields);
        return formattedResult;
    }
    async getAll(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthereumMainnetEventsGetAllDocument);
        const result = await this.client.query({
            query: query,
            variables: queryVariables,
        });
        const events = result?.data?.[userChain];
        if (events?.tokenEvents?.length) {
            const formattedResult = formatQueryResult(events, 'tokenEvents', 'tokenEventsPageInfo');
            return formattedResult;
        }
        return { results: [], pageInfo: emptyPageInfo };
    }
}
tslib.__decorate([
    ValidateInput(contractEventsValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], EventsController.prototype, "getByContract", null);
tslib.__decorate([
    ValidateInput(collectionEventsValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], EventsController.prototype, "getByNFTCollection", null);
tslib.__decorate([
    ValidateInput(nftEventsValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], EventsController.prototype, "getByNFT", null);
tslib.__decorate([
    ValidateInput(allEventsValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], EventsController.prototype, "getAll", null);

var __schema={queryType:{name:"Query"},mutationType:null,subscriptionType:null,types:[{kind:"SCALAR",name:"BigInt",description:"The `BigInt` scalar type represents non-fractional signed whole numeric values.",fields:null,inputFields:null,interfaces:null,enumValues:null,possibleTypes:null},{kind:"SCALAR",name:"Boolean",description:"The `Boolean` scalar type represents `true` or `false`.",fields:null,inputFields:null,interfaces:null,enumValues:null,possibleTypes:null},{kind:"INTERFACE",name:"Collection",description:"Fetches an NFT collection",fields:[{name:"address",description:"The contract address of the collection",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"attributes",description:"Attributes within a collection",args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"CollectionAttributesConnection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"bannerImage",description:"Collection banner image.",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TokenUpload",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"baseTokenUri",description:"The base token uri of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"circulatingSupply",description:"The circulating supply of tokens within the collection",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contract",description:null,args:[],type:{kind:"OBJECT",name:"NFTContract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"description",description:"The description of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"externalUrl",description:"The external url of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"image",description:"The collection image.",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TokenUpload",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"name",description:"The name of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"ohlcvChart",description:null,args:[{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"CollectionOhlcvChartInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionOHLCVChart",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"openseaMetadata",description:"Opensea metadata",args:[],type:{kind:"OBJECT",name:"OpenSeaMetadata",ofType:null},isDeprecated:false,deprecationReason:null},{name:"orderHistory",description:"Order history for a collection",args:[{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"CollectionOrderHistoryInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionOrderHistory",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"sales",description:"The sales of the collection",args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"CollectionSaleFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderBy",description:null,type:{kind:"ENUM",name:"CollectionSaleOrderBy",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderDirection",description:"The direction you want to order results: ASC/DESC. Defaults to DESC.",type:{kind:"ENUM",name:"OrderDirection",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"CollectionSalesConnection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"slug",description:"The slug of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"symbol",description:"The symbol of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalSupply",description:"The total supply of tokens within the collection",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"twitterUsername",description:"The twitter username of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"wallets",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionWalletsConnection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:[{kind:"OBJECT",name:"ERC721Collection",ofType:null},{kind:"OBJECT",name:"ERC1155Collection",ofType:null}]},{kind:"OBJECT",name:"CollectionAttribute",description:"Attributes within a collection",fields:[{name:"name",description:"The trait key",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"value",description:"The value of the trait",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionAttributesConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionAttributesConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionAttributesConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionAttribute",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionOHLCVChart",description:"Collection OHLCV chart stats",fields:[{name:"average",description:null,args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"close",description:null,args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"count",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Float",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"high",description:null,args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"low",description:null,args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"open",description:null,args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"timestamp",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"DateTime",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"volume",description:null,args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"ENUM",name:"CollectionOHLCVChartInterval",description:"Filter by input interval",fields:null,inputFields:null,interfaces:null,enumValues:[{name:"FIFTEEN_MINUTES",description:null,isDeprecated:false,deprecationReason:null},{name:"FIVE_MINUTES",description:null,isDeprecated:false,deprecationReason:null},{name:"ONE_DAY",description:null,isDeprecated:false,deprecationReason:null},{name:"ONE_HOUR",description:null,isDeprecated:false,deprecationReason:null},{name:"ONE_MINUTE",description:null,isDeprecated:false,deprecationReason:null},{name:"SEVEN_DAYS",description:null,isDeprecated:false,deprecationReason:null},{name:"SIX_HOURS",description:null,isDeprecated:false,deprecationReason:null},{name:"THIRTY_DAYS",description:null,isDeprecated:false,deprecationReason:null},{name:"THIRTY_MINUTES",description:null,isDeprecated:false,deprecationReason:null},{name:"TWELVE_HOURS",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"INPUT_OBJECT",name:"CollectionOhlcvChartInput",description:null,fields:null,inputFields:[{name:"confirmedAtGte",description:null,type:{kind:"SCALAR",name:"DateTime",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"confirmedAtLte",description:null,type:{kind:"SCALAR",name:"DateTime",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"interval",description:null,type:{kind:"ENUM",name:"CollectionOHLCVChartInterval",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"limit",description:null,type:{kind:"SCALAR",name:"Float",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionOrderHistory",description:"Collection order history summary",fields:[{name:"fromAddress",description:"The address of the seller",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"priceInEth",description:"The price of the transaction in the base network token",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Float",ofType:null}},isDeprecated:true,deprecationReason:"Use \"priceInNativeToken\" instead"},{name:"priceInNativeToken",description:"The price of the transaction in the base network token",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Float",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"timestamp",description:"The timestamp of the transaction",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"DateTime",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"toAddress",description:"The address of the buyer",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"tokenId",description:"The token id",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transactionHash",description:"The transaction hash",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"CollectionOrderHistoryInput",description:null,fields:null,inputFields:[{name:"confirmedAtGte",description:"The confirmedAt date is greater than or equal to, ie. 2023-01-01T00:00:00Z",type:{kind:"SCALAR",name:"DateTime",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"confirmedAtLte",description:"The confirmedAt date is less than or equal to, ie. 2023-01-01T00:00:00Z",type:{kind:"SCALAR",name:"DateTime",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"isLimited",description:"Whether the order is limited",type:{kind:"SCALAR",name:"Boolean",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionSale",description:"Sale of a collection token",fields:[{name:"estimatedConfirmedAt",description:"The estimated time the sale was confirmed",args:[],type:{kind:"SCALAR",name:"DateTime",ofType:null},isDeprecated:false,deprecationReason:null},{name:"priceInNativeToken",description:"The price in network base token",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"quantitySold",description:"The quantity sold",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"CollectionSaleFilterInput",description:"Filter input for collection sales",fields:null,inputFields:[{name:"timePeriod",description:"A time period relative to the current time in which to filter collection sales by.",type:{kind:"ENUM",name:"CollectionSalePeriod",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"timeRange",description:"Custom time range in which to filter collection sales by. Available only to paid customers.",type:{kind:"INPUT_OBJECT",name:"DateTimeInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"ENUM",name:"CollectionSaleOrderBy",description:null,fields:null,inputFields:null,interfaces:null,enumValues:[{name:"TIMESTAMP",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"ENUM",name:"CollectionSalePeriod",description:null,fields:null,inputFields:null,interfaces:null,enumValues:[{name:"FIFTEEN_MINUTES",description:null,isDeprecated:false,deprecationReason:null},{name:"FIVE_MINUTES",description:null,isDeprecated:false,deprecationReason:null},{name:"ONE_DAY",description:null,isDeprecated:false,deprecationReason:null},{name:"ONE_HOUR",description:null,isDeprecated:false,deprecationReason:null},{name:"ONE_MINUTE",description:null,isDeprecated:false,deprecationReason:null},{name:"SEVEN_DAYS",description:null,isDeprecated:false,deprecationReason:null},{name:"THIRTY_MINUTES",description:null,isDeprecated:false,deprecationReason:null},{name:"TWELVE_HOURS",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"OBJECT",name:"CollectionSalesConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionSalesConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionSalesConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionSale",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"ENUM",name:"CollectionStandard",description:"The ERC standard of the collection",fields:null,inputFields:null,interfaces:null,enumValues:[{name:"ERC721",description:null,isDeprecated:false,deprecationReason:null},{name:"ERC1155",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"INPUT_OBJECT",name:"CollectionStandardInput",description:null,fields:null,inputFields:[{name:"eq",description:"Equal to",type:{kind:"ENUM",name:"CollectionStandard",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"in",description:"In",type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"CollectionStandard",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"notIn",description:"Not In",type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"CollectionStandard",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionStats",description:"Stats of a collection",fields:[{name:"average",description:"Average sale price of the collection",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:true,deprecationReason:"Use 'averageInNativeToken' instead"},{name:"averageInNativeToken",description:"Average sale price of the collection in native token",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"ceiling",description:"Highest sale price of the collection",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:true,deprecationReason:"Use 'ceilingInNativeToken' instead"},{name:"ceilingInNativeToken",description:"Highest sale price of the collection in native token",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"floor",description:"Lowest sale price of the collection",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:true,deprecationReason:"Use 'floorInNativeToken' instead"},{name:"floorInNativeToken",description:"Lowest sale price of the collection in native token",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"totalSales",description:"Total sales of the collection",args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null},{name:"volume",description:"Total sales volume of the collection",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:true,deprecationReason:"Use 'volumeInNativeToken' instead"},{name:"volumeInNativeToken",description:"Total sales volume of the collection in native token",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionTokenEventsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionTokenEventsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionTokenEventsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"INTERFACE",name:"TokenEvent",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionWalletsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionWalletsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"CollectionWalletsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"Wallet",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"CollectionsFilterInput",description:"Filter input for collections",fields:null,inputFields:[{name:"address",description:"The address of the collection",type:{kind:"INPUT_OBJECT",name:"StringInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"ercStandard",description:"The ERC standard of the collection",type:{kind:"INPUT_OBJECT",name:"CollectionStandardInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"INTERFACE",name:"Contract",description:"Fetches a contract",fields:[{name:"abi",description:"The ABI of the contract",args:[],type:{kind:"SCALAR",name:"JSON",ofType:null},isDeprecated:false,deprecationReason:null},{name:"address",description:"The contract address",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"isVerified",description:"Contract with verified ABI",args:[],type:{kind:"SCALAR",name:"Boolean",ofType:null},isDeprecated:false,deprecationReason:null},{name:"name",description:"The name of the contract",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"supportedErcInterfaces",description:"The supported ERC interfaces of the contract",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"symbol",description:"The symbol of the contract",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ContractTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:[{kind:"OBJECT",name:"NFTContract",ofType:null},{kind:"OBJECT",name:"TokenContract",ofType:null}]},{kind:"ENUM",name:"ContractStandard",description:null,fields:null,inputFields:null,interfaces:null,enumValues:[{name:"ERC20",description:null,isDeprecated:false,deprecationReason:null},{name:"ERC721",description:null,isDeprecated:false,deprecationReason:null},{name:"ERC1155",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"INPUT_OBJECT",name:"ContractStandardInput",description:null,fields:null,inputFields:[{name:"eq",description:"Equal to",type:{kind:"ENUM",name:"ContractStandard",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"in",description:"In",type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"ContractStandard",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"notIn",description:"Not In",type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"ContractStandard",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ContractTokenEventsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ContractTokenEventsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ContractTokenEventsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"INTERFACE",name:"TokenEvent",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ContractsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ContractsEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ContractsEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"INTERFACE",name:"Contract",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"ContractsFilterInput",description:"Filter input for contracts",fields:null,inputFields:[{name:"address",description:"The address of the contract",type:{kind:"INPUT_OBJECT",name:"StringInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"SCALAR",name:"DateTime",description:"A date-time string at UTC, such as 2007-12-03T10:15:30Z, compliant with the `date-time` format outlined in section 5.6 of the RFC 3339 profile of the ISO 8601 standard for representation of dates and times using the Gregorian calendar.",fields:null,inputFields:null,interfaces:null,enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"DateTimeInput",description:null,fields:null,inputFields:[{name:"eq",description:"Equal to a date ie. 2023-01-01T00:00:00Z",type:{kind:"SCALAR",name:"DateTime",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"gt",description:"Greater than a date ie. 2023-01-01T00:00:00Z",type:{kind:"SCALAR",name:"DateTime",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"gte",description:"Greater than or equal to a date ie. 2023-01-01T00:00:00Z",type:{kind:"SCALAR",name:"DateTime",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"lt",description:"Less than a date ie. 2023-01-01T00:00:00Z",type:{kind:"SCALAR",name:"DateTime",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"lte",description:"Less than or equal to a date ie. 2023-01-01T00:00:00Z",type:{kind:"SCALAR",name:"DateTime",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ERC721Collection",description:null,fields:[{name:"address",description:"The contract address of the collection",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"attributes",description:"Attributes within a collection",args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"CollectionAttributesConnection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"bannerImage",description:"Collection banner image.",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TokenUpload",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"baseTokenUri",description:"The base token uri of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"circulatingSupply",description:"The circulating supply of tokens within the collection",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contract",description:null,args:[],type:{kind:"OBJECT",name:"NFTContract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"description",description:"The description of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"externalUrl",description:"The external url of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"image",description:"The collection image.",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TokenUpload",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"name",description:"The name of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"nft",description:null,args:[{name:"tokenId",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"ERC721NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"nfts",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"NFTsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ERC721CollectionTokensConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"ohlcvChart",description:null,args:[{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"CollectionOhlcvChartInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionOHLCVChart",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"openseaMetadata",description:"Opensea metadata",args:[],type:{kind:"OBJECT",name:"OpenSeaMetadata",ofType:null},isDeprecated:false,deprecationReason:null},{name:"orderHistory",description:"Order history for a collection",args:[{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"CollectionOrderHistoryInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionOrderHistory",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"sales",description:"The sales of the collection",args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"CollectionSaleFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderBy",description:null,type:{kind:"ENUM",name:"CollectionSaleOrderBy",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderDirection",description:"The direction you want to order results: ASC/DESC. Defaults to DESC.",type:{kind:"ENUM",name:"OrderDirection",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"CollectionSalesConnection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"slug",description:"The slug of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"stats",description:null,args:[{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"ERC721CollectionStatsInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"CollectionStats",ofType:null},isDeprecated:false,deprecationReason:null},{name:"symbol",description:"The symbol of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalSupply",description:"The total supply of tokens within the collection",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"twitterUsername",description:"The twitter username of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"wallets",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionWalletsConnection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"Collection",ofType:null}],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"ERC721CollectionStatsInput",description:null,fields:null,inputFields:[{name:"timeRange",description:null,type:{kind:"INPUT_OBJECT",name:"DateTimeInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ERC721CollectionTokensConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ERC721CollectionTokensEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ERC721CollectionTokensEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ERC721NFT",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ERC721NFT",description:null,fields:[{name:"animationUrl",description:"The animation url of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"attributes",description:"The attributes of the token.",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TokenAttribute",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"collection",description:null,args:[],type:{kind:"OBJECT",name:"ERC721Collection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"collectionSlug",description:"The slug of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contract",description:null,args:[],type:{kind:"OBJECT",name:"NFTContract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contractAddress",description:"The contract address",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"description",description:"The description of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"externalUrl",description:"The external url of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"metadata",description:"The metadata of the token",args:[],type:{kind:"SCALAR",name:"JSONObject",ofType:null},isDeprecated:false,deprecationReason:null},{name:"name",description:"The name of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"NFTTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"tokenId",description:"The token id",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"uploads",description:"The uploads of a token.",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TokenUpload",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"wallet",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"NFT",ofType:null}],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ERC1155Collection",description:null,fields:[{name:"address",description:"The contract address of the collection",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"attributes",description:"Attributes within a collection",args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"CollectionAttributesConnection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"bannerImage",description:"Collection banner image.",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TokenUpload",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"baseTokenUri",description:"The base token uri of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"circulatingSupply",description:"The circulating supply of tokens within the collection",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contract",description:null,args:[],type:{kind:"OBJECT",name:"NFTContract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"description",description:"The description of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"externalUrl",description:"The external url of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"image",description:"The collection image.",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TokenUpload",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"name",description:"The name of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"nft",description:null,args:[{name:"tokenId",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"ERC1155NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"nfts",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"NFTsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ERC1155CollectionTokensConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"ohlcvChart",description:null,args:[{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"CollectionOhlcvChartInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionOHLCVChart",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"openseaMetadata",description:"Opensea metadata",args:[],type:{kind:"OBJECT",name:"OpenSeaMetadata",ofType:null},isDeprecated:false,deprecationReason:null},{name:"orderHistory",description:"Order history for a collection",args:[{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"CollectionOrderHistoryInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionOrderHistory",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"sales",description:"The sales of the collection",args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"CollectionSaleFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderBy",description:null,type:{kind:"ENUM",name:"CollectionSaleOrderBy",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderDirection",description:"The direction you want to order results: ASC/DESC. Defaults to DESC.",type:{kind:"ENUM",name:"OrderDirection",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"CollectionSalesConnection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"slug",description:"The slug of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"stats",description:null,args:[{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"ERC1155CollectionStatsInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"CollectionStats",ofType:null},isDeprecated:false,deprecationReason:null},{name:"symbol",description:"The symbol of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalSupply",description:"The total supply of tokens within the collection",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"twitterUsername",description:"The twitter username of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"wallets",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"CollectionWalletsConnection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"Collection",ofType:null}],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"ERC1155CollectionStatsInput",description:null,fields:null,inputFields:[{name:"timeRange",description:null,type:{kind:"INPUT_OBJECT",name:"DateTimeInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ERC1155CollectionTokensConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ERC1155CollectionTokensEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ERC1155CollectionTokensEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ERC1155NFT",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ERC1155NFT",description:null,fields:[{name:"animationUrl",description:"The animation url of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"collection",description:null,args:[],type:{kind:"OBJECT",name:"ERC1155Collection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"collectionSlug",description:"The slug of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contract",description:null,args:[],type:{kind:"OBJECT",name:"NFTContract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contractAddress",description:"The contract address",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"description",description:"The description of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"externalUrl",description:"The external url of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"metadata",description:"The metadata of the token",args:[],type:{kind:"SCALAR",name:"JSONObject",ofType:null},isDeprecated:false,deprecationReason:null},{name:"name",description:"The name of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"NFTTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"tokenId",description:"The token id",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"uploads",description:"The uploads of a token.",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TokenUpload",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"wallets",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ERC1155NFTWalletsConnection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"NFT",ofType:null}],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ERC1155NFTWalletsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ERC1155NFTWalletsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"ERC1155NFTWalletsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"Wallet",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"ENUM",name:"ERCStandard",description:null,fields:null,inputFields:null,interfaces:null,enumValues:[{name:"ERC20",description:null,isDeprecated:false,deprecationReason:null},{name:"ERC721",description:null,isDeprecated:false,deprecationReason:null},{name:"ERC1155",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"OBJECT",name:"EVMSchemaType",description:null,fields:[{name:"collection",description:null,args:[{name:"contractAddress",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"INTERFACE",name:"Collection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"collections",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"CollectionsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"EVMSchemaTypeCollectionsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"contract",description:null,args:[{name:"contractAddress",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"INTERFACE",name:"Contract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contracts",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"ContractsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ContractsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"gasPrices",description:"Fetch historical gas prices by block number.",args:[{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"GasPriceFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderDirection",description:"The direction you want to order results based on blockNumber: ASC/DESC. Defaults to DESC.",type:{kind:"ENUM",name:"OrderDirection",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"GasPrice",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"nft",description:null,args:[{name:"contractAddress",description:"The address of the contract that the nft is under",type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"tokenId",description:"The id of the nft",type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"INTERFACE",name:"NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"EVMSchemaTypeTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"transaction",description:null,args:[{name:"hash",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"Transaction",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transactions",description:"Returns a list of transactions. Ordered by block number and index.",args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TransactionsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderDirection",description:"The direction you want to order results: ASC/DESC. Defaults to DESC.",type:{kind:"ENUM",name:"OrderDirection",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"EVMSchemaTypeTransactionsConnection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"trendingCollections",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TrendingCollectionsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderBy",description:null,type:{kind:"ENUM",name:"TrendingOrderBy",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderDirection",description:"The direction you want to order results: ASC/DESC. Defaults to DESC.",type:{kind:"ENUM",name:"OrderDirection",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"EVMSchemaTypeTrendingCollectionsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"walletByAddress",description:"Wallet by address",args:[{name:"address",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"walletByENS",description:"Wallet by ENS name",args:[{name:"ensName",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"EVMSchemaTypeCollectionsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"EVMSchemaTypeCollectionsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"EVMSchemaTypeCollectionsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"INTERFACE",name:"Collection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"EVMSchemaTypeTokenEventsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"EVMSchemaTypeTokenEventsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"EVMSchemaTypeTokenEventsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"INTERFACE",name:"TokenEvent",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"EVMSchemaTypeTransactionsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"EVMSchemaTypeTransactionsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"EVMSchemaTypeTransactionsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"Transaction",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"EVMSchemaTypeTrendingCollectionsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"EVMSchemaTypeTrendingCollectionsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"EVMSchemaTypeTrendingCollectionsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TrendingCollection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"SCALAR",name:"Float",description:"The `Float` scalar type represents signed double-precision fractional values as specified by [IEEE 754](https://en.wikipedia.org/wiki/IEEE_floating_point).",fields:null,inputFields:null,interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"GasPrice",description:"Gas Prices for a given block. Gas values are returned in Wei.",fields:[{name:"average",description:"The average gas price",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Float",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"blockNumber",description:"The block number",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"ceiling",description:"The highest gas price",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Float",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"floor",description:"The lowest gas price",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Float",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"median",description:"The median gas price",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Float",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"total",description:"The total gas price",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Float",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"GasPriceFilterInput",description:"Filter input for gas prices",fields:null,inputFields:[{name:"blockNumber",description:"The block number",type:{kind:"INPUT_OBJECT",name:"IntegerInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"SCALAR",name:"Int",description:"The `Int` scalar type represents non-fractional signed whole numeric values. Int can represent values between -(2^31) and 2^31 - 1.",fields:null,inputFields:null,interfaces:null,enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"IntegerInput",description:null,fields:null,inputFields:[{name:"eq",description:"Equal to",type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"gt",description:"Greater than",type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"gte",description:"Greater than or equal to",type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"in",description:"In",type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"lt",description:"Less than",type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"lte",description:"Less than or equal to",type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"SCALAR",name:"JSON",description:"The `JSON` scalar type represents JSON values as specified by [ECMA-404](http://www.ecma-international.org/publications/files/ECMA-ST/ECMA-404.pdf).",fields:null,inputFields:null,interfaces:null,enumValues:null,possibleTypes:null},{kind:"SCALAR",name:"JSONObject",description:"The `JSONObject` scalar type represents JSON objects as specified by [ECMA-404](http://www.ecma-international.org/publications/files/ECMA-ST/ECMA-404.pdf).",fields:null,inputFields:null,interfaces:null,enumValues:null,possibleTypes:null},{kind:"ENUM",name:"Marketplace",description:"Marketplace where the token was sold",fields:null,inputFields:null,interfaces:null,enumValues:[{name:"BLUR",description:null,isDeprecated:false,deprecationReason:null},{name:"CRYPTOPUNKS",description:null,isDeprecated:false,deprecationReason:null},{name:"LOOKSRARE",description:null,isDeprecated:false,deprecationReason:null},{name:"NIFTY_GATEWAY",description:null,isDeprecated:false,deprecationReason:null},{name:"OPENSEA",description:null,isDeprecated:false,deprecationReason:null},{name:"SEAPORT",description:null,isDeprecated:false,deprecationReason:null},{name:"X2Y2",description:null,isDeprecated:false,deprecationReason:null},{name:"ZEROX",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"INPUT_OBJECT",name:"MarketplaceInput",description:null,fields:null,inputFields:[{name:"eq",description:null,type:{kind:"ENUM",name:"Marketplace",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"in",description:null,type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"Marketplace",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"notIn",description:null,type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"Marketplace",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"INTERFACE",name:"NFT",description:null,fields:[{name:"animationUrl",description:"The animation url of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"collection",description:null,args:[],type:{kind:"INTERFACE",name:"Collection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"collectionSlug",description:"The slug of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contract",description:null,args:[],type:{kind:"OBJECT",name:"NFTContract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contractAddress",description:"The contract address",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"description",description:"The description of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"externalUrl",description:"The external url of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"metadata",description:"The metadata of the token",args:[],type:{kind:"SCALAR",name:"JSONObject",ofType:null},isDeprecated:false,deprecationReason:null},{name:"name",description:"The name of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"NFTTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"tokenId",description:"The token id",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"uploads",description:"The uploads of a token.",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"TokenUpload",ofType:null}}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:[{kind:"OBJECT",name:"ERC721NFT",ofType:null},{kind:"OBJECT",name:"ERC1155NFT",ofType:null}]},{kind:"OBJECT",name:"NFTContract",description:null,fields:[{name:"abi",description:"The ABI of the contract",args:[],type:{kind:"SCALAR",name:"JSON",ofType:null},isDeprecated:false,deprecationReason:null},{name:"address",description:"The contract address",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"collection",description:null,args:[],type:{kind:"INTERFACE",name:"Collection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"isVerified",description:"Contract with verified ABI",args:[],type:{kind:"SCALAR",name:"Boolean",ofType:null},isDeprecated:false,deprecationReason:null},{name:"name",description:"The name of the contract",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"supportedErcInterfaces",description:"The supported ERC interfaces of the contract",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"symbol",description:"The symbol of the contract",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ContractTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"Contract",ofType:null}],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"NFTTokenEventsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"NFTTokenEventsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"NFTTokenEventsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"INTERFACE",name:"TokenEvent",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"NFTWallet",description:null,fields:[{name:"owner",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"walletAddress",description:"The address of the wallet",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"NFTsFilterInput",description:null,fields:null,inputFields:[{name:"contractAddressIn",description:"Contract address in",type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"OpenSeaMetadata",description:"Metadata provided by opensea",fields:[{name:"isHidden",description:"Collection is hidden on Opensea.",args:[],type:{kind:"SCALAR",name:"Boolean",ofType:null},isDeprecated:false,deprecationReason:null},{name:"isVerified",description:"Collection verified by Opensea.",args:[],type:{kind:"SCALAR",name:"Boolean",ofType:null},isDeprecated:false,deprecationReason:null},{name:"unsafeSlug",description:"Slug provided by Opensea (it might be stale).",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"ENUM",name:"OrderDirection",description:"Sort ascending (A-Z) or descending (Z-A)",fields:null,inputFields:null,interfaces:null,enumValues:[{name:"ASC",description:null,isDeprecated:false,deprecationReason:null},{name:"DESC",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"OBJECT",name:"PageInfo",description:null,fields:[{name:"endCursor",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"hasNextPage",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Boolean",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"hasPreviousPage",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Boolean",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"startCursor",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"Query",description:null,fields:[{name:"ethereum",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"EVMSchemaType",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"ethereumSepolia",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"EVMSchemaType",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"polygon",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"EVMSchemaType",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"SCALAR",name:"String",description:"The `String` scalar type represents textual data, represented as UTF-8 character sequences. The String type is most often used by GraphQL to represent free-form human-readable text.",fields:null,inputFields:null,interfaces:null,enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"StringInput",description:null,fields:null,inputFields:[{name:"eq",description:"Equal to",type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"in",description:"In",type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"notIn",description:"Not In",type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TokenAttribute",description:"A token attribute",fields:[{name:"name",description:"Attribute name",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"value",description:"Attribute value",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TokenBurnEvent",description:"A token burn event",fields:[{name:"blockNumber",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"contract",description:null,args:[],type:{kind:"INTERFACE",name:"Contract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contractAddress",description:"The address of the token contract",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"contractERCStandard",description:"The ERC standard of the token contract",args:[],type:{kind:"ENUM",name:"ERCStandard",ofType:null},isDeprecated:false,deprecationReason:null},{name:"from",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"fromAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"nft",description:null,args:[],type:{kind:"INTERFACE",name:"NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"timestamp",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"DateTime",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"to",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"toAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"tokenId",description:"The token id",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenQuantity",description:"The quantity",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"transaction",description:null,args:[],type:{kind:"OBJECT",name:"Transaction",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transactionHash",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transferIndex",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"type",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"TokenTransferType",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"TokenEvent",ofType:null}],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TokenContract",description:null,fields:[{name:"abi",description:"The ABI of the contract",args:[],type:{kind:"SCALAR",name:"JSON",ofType:null},isDeprecated:false,deprecationReason:null},{name:"address",description:"The contract address",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"decimals",description:null,args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"details",description:null,args:[],type:{kind:"OBJECT",name:"TokenDetailsType",ofType:null},isDeprecated:false,deprecationReason:null},{name:"isVerified",description:"Contract with verified ABI",args:[],type:{kind:"SCALAR",name:"Boolean",ofType:null},isDeprecated:false,deprecationReason:null},{name:"name",description:"The name of the contract",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"supportedErcInterfaces",description:"The supported ERC interfaces of the contract",args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"symbol",description:"The symbol of the contract",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"ContractTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"Contract",ofType:null}],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TokenDetailsType",description:"Token details",fields:[{name:"address",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"name",description:"The name of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"slug",description:"The slug of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"symbol",description:"The symbol of the token",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"INTERFACE",name:"TokenEvent",description:null,fields:[{name:"blockNumber",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"from",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"fromAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"timestamp",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"DateTime",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"to",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"toAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"transaction",description:null,args:[],type:{kind:"OBJECT",name:"Transaction",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transactionHash",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transferIndex",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"type",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"TokenTransferType",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:[{kind:"OBJECT",name:"TokenBurnEvent",ofType:null},{kind:"OBJECT",name:"TokenMintEvent",ofType:null},{kind:"OBJECT",name:"TokenSaleEvent",ofType:null},{kind:"OBJECT",name:"TokenSwapEvent",ofType:null},{kind:"OBJECT",name:"TokenTransferEvent",ofType:null}]},{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",description:"Filter input for token events",fields:null,inputFields:[{name:"blockNumber",description:"Filter token events by their block number",type:{kind:"INPUT_OBJECT",name:"IntegerInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"contractAddress",description:"Filter token events by contract address",type:{kind:"INPUT_OBJECT",name:"StringInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"contractStandard",description:"Filter token events by their contract standard",type:{kind:"INPUT_OBJECT",name:"ContractStandardInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"fromAddress",description:"Filter token events by the \"from\" wallet",type:{kind:"INPUT_OBJECT",name:"StringInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"marketplace",description:"Filter token events by the marketplace. It's only valid for SALE types",type:{kind:"INPUT_OBJECT",name:"MarketplaceInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"timestamp",description:"Filter token events by their estimated confirmation date",type:{kind:"INPUT_OBJECT",name:"DateTimeInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"toAddress",description:"Filter token events by the \"to\" wallet",type:{kind:"INPUT_OBJECT",name:"StringInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"transactionHash",description:"Filter token events by transaction hash",type:{kind:"INPUT_OBJECT",name:"StringInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"type",description:"Filter token events by their type",type:{kind:"INPUT_OBJECT",name:"TokenTransferTypeInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"walletAddress",description:"Filter token events by the \"to\" and \"from\" wallet",type:{kind:"INPUT_OBJECT",name:"StringInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TokenMintEvent",description:"A token mint event",fields:[{name:"blockNumber",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"contract",description:null,args:[],type:{kind:"INTERFACE",name:"Contract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contractAddress",description:"The address of the token contract",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"contractERCStandard",description:"The ERC standard of the token contract",args:[],type:{kind:"ENUM",name:"ERCStandard",ofType:null},isDeprecated:false,deprecationReason:null},{name:"from",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"fromAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"nft",description:null,args:[],type:{kind:"INTERFACE",name:"NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"timestamp",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"DateTime",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"to",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"toAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"tokenId",description:"The token id",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenQuantity",description:"The quantity",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"transaction",description:null,args:[],type:{kind:"OBJECT",name:"Transaction",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transactionHash",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transferIndex",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"type",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"TokenTransferType",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"TokenEvent",ofType:null}],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TokenSaleEvent",description:"A token sale event",fields:[{name:"blockNumber",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"contractAddress",description:"The address of the token contract",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"contractERCStandard",description:"The ERC standard of the token contract",args:[],type:{kind:"ENUM",name:"ERCStandard",ofType:null},isDeprecated:false,deprecationReason:null},{name:"from",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"fromAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"marketplace",description:"Marketplace used to make the sale",args:[],type:{kind:"ENUM",name:"Marketplace",ofType:null},isDeprecated:false,deprecationReason:null},{name:"receivedNft",description:null,args:[],type:{kind:"INTERFACE",name:"NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"receivedTokenContract",description:null,args:[],type:{kind:"INTERFACE",name:"Contract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"receivedTokenContractAddress",description:"The address of the token received as payment of the sale proceeds",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"receivedTokenId",description:"The id of the token received as payment of the sale proceeds",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"receivedTokenQuantity",description:"The quantity of tokens received as payment of the sale proceeds",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"sentNft",description:null,args:[],type:{kind:"INTERFACE",name:"NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"sentTokenContract",description:null,args:[],type:{kind:"INTERFACE",name:"Contract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"sentTokenId",description:"The token id",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"sentTokenQuantity",description:"The quantity",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"timestamp",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"DateTime",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"to",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"toAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"transaction",description:null,args:[],type:{kind:"OBJECT",name:"Transaction",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transactionHash",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transferIndex",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"type",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"TokenTransferType",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"TokenEvent",ofType:null}],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TokenSwapEvent",description:"A token swap event",fields:[{name:"blockNumber",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"from",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"fromAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"receivedNft",description:null,args:[],type:{kind:"INTERFACE",name:"NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"receivedTokenContract",description:null,args:[],type:{kind:"INTERFACE",name:"Contract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"receivedTokenContractAddress",description:"The address of the token received",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"receivedTokenId",description:"The id of the token received",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"receivedTokenQuantity",description:"The quantity",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"sentNft",description:null,args:[],type:{kind:"INTERFACE",name:"NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"sentTokenContract",description:null,args:[],type:{kind:"INTERFACE",name:"Contract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"sentTokenContractAddress",description:"The address of the token contract",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"sentTokenContractERCStandard",description:"The ERC standard of the token contract",args:[],type:{kind:"ENUM",name:"ERCStandard",ofType:null},isDeprecated:false,deprecationReason:null},{name:"sentTokenId",description:"The token id",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"sentTokenQuantity",description:"The quantity",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"timestamp",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"DateTime",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"to",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"toAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"transaction",description:null,args:[],type:{kind:"OBJECT",name:"Transaction",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transactionHash",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transferIndex",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"type",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"TokenTransferType",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"TokenEvent",ofType:null}],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TokenTransferEvent",description:"A token transfer event",fields:[{name:"blockNumber",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"contract",description:null,args:[],type:{kind:"INTERFACE",name:"Contract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contractAddress",description:"The address of the token contract",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"contractERCStandard",description:"The ERC standard of the token contract",args:[],type:{kind:"ENUM",name:"ERCStandard",ofType:null},isDeprecated:false,deprecationReason:null},{name:"from",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"fromAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"nft",description:null,args:[],type:{kind:"INTERFACE",name:"NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"timestamp",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"DateTime",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"to",description:null,args:[],type:{kind:"OBJECT",name:"Wallet",ofType:null},isDeprecated:false,deprecationReason:null},{name:"toAddress",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"tokenId",description:"The token id",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenQuantity",description:"The quantity",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"transaction",description:null,args:[],type:{kind:"OBJECT",name:"Transaction",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transactionHash",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transferIndex",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"type",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"TokenTransferType",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[{kind:"INTERFACE",name:"TokenEvent",ofType:null}],enumValues:null,possibleTypes:null},{kind:"ENUM",name:"TokenTransferType",description:null,fields:null,inputFields:null,interfaces:null,enumValues:[{name:"BURN",description:null,isDeprecated:false,deprecationReason:null},{name:"MINT",description:null,isDeprecated:false,deprecationReason:null},{name:"SALE",description:null,isDeprecated:false,deprecationReason:null},{name:"SWAP",description:null,isDeprecated:false,deprecationReason:null},{name:"TRANSFER",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"INPUT_OBJECT",name:"TokenTransferTypeInput",description:null,fields:null,inputFields:[{name:"eq",description:null,type:{kind:"ENUM",name:"TokenTransferType",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"in",description:null,type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"TokenTransferType",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"notIn",description:null,type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"TokenTransferType",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TokenUpload",description:"Token media uploads.",fields:[{name:"height",description:"The upload height.",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"mimeType",description:"The upload mimeType.",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"url",description:"The upload url.",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"width",description:"The upload width.",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"Transaction",description:null,fields:[{name:"blockNumber",description:"The block number",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"blockTimestamp",description:"The block timestamp",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"DateTime",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"contractAddress",description:"The contract address",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"cumulativeGasUsed",description:"The cumulative gas used by this transaction",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"effectiveGasPrice",description:"The effective gas price in Gwei",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"fromAddress",description:"The from address",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"gas",description:"The amount of gas supplied for this transaction to happen",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"gasPrice",description:"Cost in Gwei per unit of gas for this transaction",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"gasUsed",description:"The amount of gas used by this transaction",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"hash",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"input",description:"The transaction input",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"maxFeePerGas",description:"Max gas fee in Gwei",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"maxPriorityFeePerGas",description:"Max gas priority fee in Gwei",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null},{name:"toAddress",description:"The to address",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"transactionIndex",description:"The transaction index",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Int",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"type",description:"The transaction type",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"value",description:"The transaction value",args:[],type:{kind:"SCALAR",name:"BigInt",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"TransactionsFilterInput",description:"Filter input for transactions",fields:null,inputFields:[{name:"blockNumber",description:"The block number",type:{kind:"INPUT_OBJECT",name:"IntegerInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"fromAddress",description:"The from address",type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"timestamp",description:"The timestamp",type:{kind:"INPUT_OBJECT",name:"DateTimeInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"toAddress",description:"The to address",type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TrendingCollection",description:null,fields:[{name:"collection",description:"The collection",args:[],type:{kind:"INTERFACE",name:"Collection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"stats",description:null,args:[],type:{kind:"OBJECT",name:"TrendingCollectionStats",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"TrendingCollectionStats",description:"Stats of a trending collection",fields:[{name:"average",description:"Average sale price of the collection",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:true,deprecationReason:"Use 'averageInNativeToken' instead"},{name:"averageInNativeToken",description:"Average sale price of the collection in native token",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"ceiling",description:"Highest sale price of the collection",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:true,deprecationReason:"Use 'ceilingInNativeToken' instead"},{name:"ceilingInNativeToken",description:"Highest sale price of the collection in native token",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"floor",description:"Lowest sale price of the collection",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:true,deprecationReason:"Use 'floorInNativeToken' instead"},{name:"floorInNativeToken",description:"Lowest sale price of the collection in native token",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"totalSales",description:"Total sales of the collection",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null},{name:"volume",description:"Total sales volume of the collection",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:true,deprecationReason:"Use 'volumeInNativeToken' instead"},{name:"volumeInNativeToken",description:"Total sales volume of the collection in native token",args:[],type:{kind:"SCALAR",name:"Float",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"TrendingCollectionsFilterInput",description:"Filter input for trending collections",fields:null,inputFields:[{name:"timePeriod",description:"A time period relative to the current time in which to filter trending collections by.",type:{kind:"ENUM",name:"TrendingPeriod",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"timeRange",description:"Custom time range in which to filter trending collections by. Available only to paid customers.",type:{kind:"INPUT_OBJECT",name:"DateTimeInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"ENUM",name:"TrendingOrderBy",description:null,fields:null,inputFields:null,interfaces:null,enumValues:[{name:"AVERAGE",description:null,isDeprecated:false,deprecationReason:null},{name:"SALES",description:null,isDeprecated:false,deprecationReason:null},{name:"VOLUME",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"ENUM",name:"TrendingPeriod",description:null,fields:null,inputFields:null,interfaces:null,enumValues:[{name:"FIFTEEN_MINUTES",description:null,isDeprecated:false,deprecationReason:null},{name:"FIVE_MINUTES",description:null,isDeprecated:false,deprecationReason:null},{name:"ONE_DAY",description:null,isDeprecated:false,deprecationReason:null},{name:"ONE_HOUR",description:null,isDeprecated:false,deprecationReason:null},{name:"ONE_MINUTE",description:null,isDeprecated:false,deprecationReason:null},{name:"SEVEN_DAYS",description:null,isDeprecated:false,deprecationReason:null},{name:"THIRTY_MINUTES",description:null,isDeprecated:false,deprecationReason:null},{name:"TWELVE_HOURS",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"OBJECT",name:"Wallet",description:null,fields:[{name:"address",description:"The address of the wallet",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"ensName",description:"The ENS name of the wallet",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"tokenBalances",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderBy",description:null,type:{kind:"ENUM",name:"WalletTokenBalanceOrder",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderDirection",description:"The direction you want to order results: ASC/DESC. Defaults to DESC.",type:{kind:"ENUM",name:"OrderDirection",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletTokenBalancesConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"tokenEvents",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TokenEventsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletTokenEventsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"transactions",description:"Returns a list of transactions this wallet is associated with. Ordered by block number and index.",args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"TransactionsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderDirection",description:"The direction you want to order results: ASC/DESC. Defaults to DESC.",type:{kind:"ENUM",name:"OrderDirection",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletTransactionsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"walletCollection",description:null,args:[{name:"collectionAddress",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"WalletCollection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"walletCollections",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"WalletCollectionsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderBy",description:null,type:{kind:"ENUM",name:"WalletCollectionOrderBy",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderDirection",description:"The direction you want to order results: ASC/DESC. Defaults to DESC.",type:{kind:"ENUM",name:"OrderDirection",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletWalletCollectionsConnection",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"walletNFT",description:null,args:[{name:"contractAddress",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"tokenId",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"OBJECT",name:"WalletNFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"walletNFTs",description:null,args:[{name:"after",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"before",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"filter",description:null,type:{kind:"INPUT_OBJECT",name:"WalletNFTsFilterInput",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"first",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"last",description:null,type:{kind:"SCALAR",name:"Int",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderBy",description:null,type:{kind:"ENUM",name:"WalletNFTsOrderBy",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"orderDirection",description:"The direction you want to order results: ASC/DESC. Defaults to DESC.",type:{kind:"ENUM",name:"OrderDirection",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletNFTsConnection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletCollection",description:null,fields:[{name:"collection",description:null,args:[],type:{kind:"INTERFACE",name:"Collection",ofType:null},isDeprecated:false,deprecationReason:null},{name:"collectionAddress",description:"The address of the collection",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"nftsCount",description:"The number of NFTs from this collection within the wallet",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"ENUM",name:"WalletCollectionOrderBy",description:null,fields:null,inputFields:null,interfaces:null,enumValues:[{name:"DATE_ACQUIRED",description:null,isDeprecated:false,deprecationReason:null},{name:"NAME",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"INPUT_OBJECT",name:"WalletCollectionsFilterInput",description:"Filter of collections in a wallet",fields:null,inputFields:[{name:"contractAddressIn",description:null,type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletNFT",description:null,fields:[{name:"nft",description:null,args:[],type:{kind:"INTERFACE",name:"NFT",ofType:null},isDeprecated:false,deprecationReason:null},{name:"nftsCount",description:"The number of NFTs from this collection within the wallet",args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletNFTsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletNFTsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletNFTsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletNFT",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"WalletNFTsContractTokenFilterInput",description:"Filter of nfts in a wallet by contract address and optional token id. Results are returned if any of the conditions are matched.",fields:null,inputFields:[{name:"contractAddress",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null},{name:"tokenId",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"INPUT_OBJECT",name:"WalletNFTsFilterInput",description:"Filter of nfts in a wallet",fields:null,inputFields:[{name:"contractTokens",description:null,type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"INPUT_OBJECT",name:"WalletNFTsContractTokenFilterInput",ofType:null}}},defaultValue:null,isDeprecated:false,deprecationReason:null}],interfaces:null,enumValues:null,possibleTypes:null},{kind:"ENUM",name:"WalletNFTsOrderBy",description:null,fields:null,inputFields:null,interfaces:null,enumValues:[{name:"DATE_ACQUIRED",description:null,isDeprecated:false,deprecationReason:null},{name:"NAME",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"OBJECT",name:"WalletTokenBalance",description:null,fields:[{name:"contract",description:null,args:[],type:{kind:"OBJECT",name:"TokenContract",ofType:null},isDeprecated:false,deprecationReason:null},{name:"contractAddress",description:"The address of the token",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalBalance",description:"The total balance of the token",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"BigInt",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"ENUM",name:"WalletTokenBalanceOrder",description:"Sort wallet token balance",fields:null,inputFields:null,interfaces:null,enumValues:[{name:"CONTRACT_ADDRESS",description:null,isDeprecated:false,deprecationReason:null},{name:"NAME",description:null,isDeprecated:false,deprecationReason:null},{name:"SYMBOL",description:null,isDeprecated:false,deprecationReason:null},{name:"TOTAL_BALANCE",description:null,isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"OBJECT",name:"WalletTokenBalancesConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletTokenBalancesConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletTokenBalancesConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletTokenBalance",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletTokenEventsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletTokenEventsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletTokenEventsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"INTERFACE",name:"TokenEvent",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletTransactionsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletTransactionsEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletTransactionsEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"Transaction",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletWalletCollectionsConnection",description:null,fields:[{name:"edges",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletWalletCollectionsConnectionEdge",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"pageInfo",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"PageInfo",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"totalCount",description:null,args:[],type:{kind:"SCALAR",name:"Int",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"WalletWalletCollectionsConnectionEdge",description:null,fields:[{name:"cursor",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"node",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"WalletCollection",ofType:null}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"__Directive",description:"A Directive provides a way to describe alternate runtime execution and type validation behavior in a GraphQL document.\n\nIn some cases, you need to provide options to alter GraphQL's execution behavior in ways field arguments will not suffice, such as conditionally including or skipping a field. Directives provide this by describing additional information to the executor.",fields:[{name:"name",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"description",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"isRepeatable",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Boolean",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"locations",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"__DirectiveLocation",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"args",description:null,args:[{name:"includeDeprecated",description:null,type:{kind:"SCALAR",name:"Boolean",ofType:null},defaultValue:"false",isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__InputValue",ofType:null}}}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"ENUM",name:"__DirectiveLocation",description:"A Directive can be adjacent to many parts of the GraphQL language, a __DirectiveLocation describes one such possible adjacencies.",fields:null,inputFields:null,interfaces:null,enumValues:[{name:"QUERY",description:"Location adjacent to a query operation.",isDeprecated:false,deprecationReason:null},{name:"MUTATION",description:"Location adjacent to a mutation operation.",isDeprecated:false,deprecationReason:null},{name:"SUBSCRIPTION",description:"Location adjacent to a subscription operation.",isDeprecated:false,deprecationReason:null},{name:"FIELD",description:"Location adjacent to a field.",isDeprecated:false,deprecationReason:null},{name:"FRAGMENT_DEFINITION",description:"Location adjacent to a fragment definition.",isDeprecated:false,deprecationReason:null},{name:"FRAGMENT_SPREAD",description:"Location adjacent to a fragment spread.",isDeprecated:false,deprecationReason:null},{name:"INLINE_FRAGMENT",description:"Location adjacent to an inline fragment.",isDeprecated:false,deprecationReason:null},{name:"VARIABLE_DEFINITION",description:"Location adjacent to a variable definition.",isDeprecated:false,deprecationReason:null},{name:"SCHEMA",description:"Location adjacent to a schema definition.",isDeprecated:false,deprecationReason:null},{name:"SCALAR",description:"Location adjacent to a scalar definition.",isDeprecated:false,deprecationReason:null},{name:"OBJECT",description:"Location adjacent to an object type definition.",isDeprecated:false,deprecationReason:null},{name:"FIELD_DEFINITION",description:"Location adjacent to a field definition.",isDeprecated:false,deprecationReason:null},{name:"ARGUMENT_DEFINITION",description:"Location adjacent to an argument definition.",isDeprecated:false,deprecationReason:null},{name:"INTERFACE",description:"Location adjacent to an interface definition.",isDeprecated:false,deprecationReason:null},{name:"UNION",description:"Location adjacent to a union definition.",isDeprecated:false,deprecationReason:null},{name:"ENUM",description:"Location adjacent to an enum definition.",isDeprecated:false,deprecationReason:null},{name:"ENUM_VALUE",description:"Location adjacent to an enum value definition.",isDeprecated:false,deprecationReason:null},{name:"INPUT_OBJECT",description:"Location adjacent to an input object type definition.",isDeprecated:false,deprecationReason:null},{name:"INPUT_FIELD_DEFINITION",description:"Location adjacent to an input object field definition.",isDeprecated:false,deprecationReason:null}],possibleTypes:null},{kind:"OBJECT",name:"__EnumValue",description:"One possible value for a given Enum. Enum values are unique values, not a placeholder for a string or numeric value. However an Enum value is returned in a JSON response as a string.",fields:[{name:"name",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"description",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"isDeprecated",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Boolean",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"deprecationReason",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"__Field",description:"Object and Interface types are described by a list of Fields, each of which has a name, potentially a list of arguments, and a return type.",fields:[{name:"name",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"description",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"args",description:null,args:[{name:"includeDeprecated",description:null,type:{kind:"SCALAR",name:"Boolean",ofType:null},defaultValue:"false",isDeprecated:false,deprecationReason:null}],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__InputValue",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"type",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__Type",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"isDeprecated",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Boolean",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"deprecationReason",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"__InputValue",description:"Arguments provided to Fields or Directives and the input fields of an InputObject are represented as Input Values which describe their type and optionally a default value.",fields:[{name:"name",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"description",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"type",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__Type",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"defaultValue",description:"A GraphQL-formatted string representing the default value for this input value.",args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"isDeprecated",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Boolean",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"deprecationReason",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"__Schema",description:"A GraphQL Schema defines the capabilities of a GraphQL server. It exposes all available types and directives on the server, as well as the entry points for query, mutation, and subscription operations.",fields:[{name:"description",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"types",description:"A list of all types supported by this server.",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__Type",ofType:null}}}},isDeprecated:false,deprecationReason:null},{name:"queryType",description:"The type that query operations will be rooted at.",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__Type",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"mutationType",description:"If this server supports mutation, the type that mutation operations will be rooted at.",args:[],type:{kind:"OBJECT",name:"__Type",ofType:null},isDeprecated:false,deprecationReason:null},{name:"subscriptionType",description:"If this server support subscription, the type that subscription operations will be rooted at.",args:[],type:{kind:"OBJECT",name:"__Type",ofType:null},isDeprecated:false,deprecationReason:null},{name:"directives",description:"A list of all directives supported by this server.",args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__Directive",ofType:null}}}},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"OBJECT",name:"__Type",description:"The fundamental unit of any GraphQL Schema is the type. There are many kinds of types in GraphQL as represented by the `__TypeKind` enum.\n\nDepending on the kind of a type, certain fields describe information about that type. Scalar types provide no information beyond a name, description and optional `specifiedByURL`, while Enum types provide their values. Object and Interface types provide the fields they describe. Abstract types, Union and Interface, provide the Object types possible at runtime. List and NonNull types compose other types.",fields:[{name:"kind",description:null,args:[],type:{kind:"NON_NULL",name:null,ofType:{kind:"ENUM",name:"__TypeKind",ofType:null}},isDeprecated:false,deprecationReason:null},{name:"name",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"description",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"specifiedByURL",description:null,args:[],type:{kind:"SCALAR",name:"String",ofType:null},isDeprecated:false,deprecationReason:null},{name:"fields",description:null,args:[{name:"includeDeprecated",description:null,type:{kind:"SCALAR",name:"Boolean",ofType:null},defaultValue:"false",isDeprecated:false,deprecationReason:null}],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__Field",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"interfaces",description:null,args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__Type",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"possibleTypes",description:null,args:[],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__Type",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"enumValues",description:null,args:[{name:"includeDeprecated",description:null,type:{kind:"SCALAR",name:"Boolean",ofType:null},defaultValue:"false",isDeprecated:false,deprecationReason:null}],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__EnumValue",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"inputFields",description:null,args:[{name:"includeDeprecated",description:null,type:{kind:"SCALAR",name:"Boolean",ofType:null},defaultValue:"false",isDeprecated:false,deprecationReason:null}],type:{kind:"LIST",name:null,ofType:{kind:"NON_NULL",name:null,ofType:{kind:"OBJECT",name:"__InputValue",ofType:null}}},isDeprecated:false,deprecationReason:null},{name:"ofType",description:null,args:[],type:{kind:"OBJECT",name:"__Type",ofType:null},isDeprecated:false,deprecationReason:null}],inputFields:null,interfaces:[],enumValues:null,possibleTypes:null},{kind:"ENUM",name:"__TypeKind",description:"An enum describing what kind of type a given `__Type` is.",fields:null,inputFields:null,interfaces:null,enumValues:[{name:"SCALAR",description:"Indicates this type is a scalar.",isDeprecated:false,deprecationReason:null},{name:"OBJECT",description:"Indicates this type is an object. `fields` and `interfaces` are valid fields.",isDeprecated:false,deprecationReason:null},{name:"INTERFACE",description:"Indicates this type is an interface. `fields`, `interfaces`, and `possibleTypes` are valid fields.",isDeprecated:false,deprecationReason:null},{name:"UNION",description:"Indicates this type is a union. `possibleTypes` is a valid field.",isDeprecated:false,deprecationReason:null},{name:"ENUM",description:"Indicates this type is an enum. `enumValues` is a valid field.",isDeprecated:false,deprecationReason:null},{name:"INPUT_OBJECT",description:"Indicates this type is an input object. `inputFields` is a valid field.",isDeprecated:false,deprecationReason:null},{name:"LIST",description:"Indicates this type is a list. `ofType` is a valid field.",isDeprecated:false,deprecationReason:null},{name:"NON_NULL",description:"Indicates this type is a non-null. `ofType` is a valid field.",isDeprecated:false,deprecationReason:null}],possibleTypes:null}],directives:[{name:"defer",description:null,isRepeatable:false,locations:["FRAGMENT_SPREAD","INLINE_FRAGMENT"],args:[{name:"if",description:null,type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Boolean",ofType:null}},defaultValue:"true",isDeprecated:false,deprecationReason:null},{name:"label",description:null,type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:null,isDeprecated:false,deprecationReason:null}]},{name:"deprecated",description:"Marks an element of a GraphQL schema as no longer supported.",isRepeatable:false,locations:["ARGUMENT_DEFINITION","ENUM_VALUE","FIELD_DEFINITION","INPUT_FIELD_DEFINITION"],args:[{name:"reason",description:"Explains why this element was deprecated, usually also including a suggestion for how to access supported similar data. Formatted using the Markdown syntax, as specified by [CommonMark](https://commonmark.org/).",type:{kind:"SCALAR",name:"String",ofType:null},defaultValue:"\"No longer supported\"",isDeprecated:false,deprecationReason:null}]},{name:"include",description:"Directs the executor to include this field or fragment only when the `if` argument is true.",isRepeatable:false,locations:["FIELD","FRAGMENT_SPREAD","INLINE_FRAGMENT"],args:[{name:"if",description:"Included when true.",type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Boolean",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}]},{name:"skip",description:"Directs the executor to skip this field or fragment when the `if` argument is true.",isRepeatable:false,locations:["FIELD","FRAGMENT_SPREAD","INLINE_FRAGMENT"],args:[{name:"if",description:"Skipped when true.",type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"Boolean",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}]},{name:"specifiedBy",description:"Exposes a URL that specifies the behavior of this scalar.",isRepeatable:false,locations:["SCALAR"],args:[{name:"url",description:"The URL that specifies the behavior of this scalar.",type:{kind:"NON_NULL",name:null,ofType:{kind:"SCALAR",name:"String",ofType:null}},defaultValue:null,isDeprecated:false,deprecationReason:null}]}]};var schema = {__schema:__schema};

const transactionsBySearchValidator = baseTransactionsInput
    .merge(supportedChainInput)
    .strict();

const transactionsByHashValidator = zod.z
    .object({
    hash: isEvmTransactionHash,
})
    .merge(supportedChainInput)
    .strict();

class TransactionsController {
    constructor(client, defaultChain = DEFAULT_CHAIN) {
        this.client = client;
        this.defaultChain = defaultChain;
    }
    async getByWallet(variables) {
        const { address, ...allVariables } = variables;
        let queryResult;
        if (isValidENSAddress(address)) {
            queryResult = await this.getByWalletENS({
                ensName: address,
                ...allVariables,
            });
        }
        else {
            queryResult = await this.getByWalletAddress({
                address,
                ...allVariables,
            });
        }
        if (!queryResult?.transactions?.length) {
            // Address can still be valid address, but not have any transactions
            const address = queryResult?.address || '';
            const ensName = queryResult?.ensName || '';
            return {
                address: address,
                ensName: ensName,
                results: [],
                pageInfo: emptyPageInfo,
            };
        }
        const formattedResult = formatQueryResult(queryResult, 'transactions', 'transactionsPageInfo');
        return formattedResult;
    }
    async getByWalletAddress(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetTransactionsByWalletAddressDocument);
        const result = await this.client.query({
            variables: queryVariables,
            query: query,
        });
        const walletByAddress = result?.data?.[userChain]?.walletByAddress;
        return walletByAddress;
    }
    async getByWalletENS(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetTransactionsByWalletENSDocument);
        const result = await this.client.query({
            variables: queryVariables,
            query: query,
        });
        const walletByENS = result?.data?.[userChain]?.walletByENS;
        return walletByENS;
    }
    async getAll(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetTransactionsBySearchDocument);
        const result = await this.client.query({
            variables: queryVariables,
            query: query,
        });
        const transactions = result?.data?.[userChain];
        if (transactions && transactions?.transactions?.length > 0) {
            const formattedResult = formatQueryResult(transactions, 'transactions', 'transactionsPageInfo');
            return formattedResult;
        }
        return {
            results: [],
            pageInfo: emptyPageInfo,
        };
    }
    async getByHash(variables) {
        const { chain, ...queryVariables } = variables;
        const userChain = chain || this.defaultChain;
        const query = modifyQueryForChain(userChain, CodegenEthMainnetTransactionsByHashDocument);
        const result = await this.client.query({
            variables: queryVariables,
            query: query,
        });
        const transaction = result?.data?.[userChain];
        if (transaction?.transaction)
            return transaction;
        return { transaction: null };
    }
}
tslib.__decorate([
    ValidateInput(balancesByWalletAddressValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], TransactionsController.prototype, "getByWallet", null);
tslib.__decorate([
    ValidateInput(transactionsBySearchValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], TransactionsController.prototype, "getAll", null);
tslib.__decorate([
    ValidateInput(transactionsByHashValidator),
    tslib.__metadata("design:type", Function),
    tslib.__metadata("design:paramtypes", [Object]),
    tslib.__metadata("design:returntype", Promise)
], TransactionsController.prototype, "getByHash", null);

var name="@quicknode/sdk";var repository={type:"git",url:"https://github.com/quiknode-labs/qn-oss.git",directory:"packages/libs/sdk"};var license="MIT";var version="1.1.1";var main="./cjs/index.js";var module$1="./esm/src/index.js";var types="./index.d.ts";var sideEffects=false;var dependencies={"@urql/core":"^4.0.7","@urql/exchange-graphcache":"^6.0.4","cross-fetch":"^3.1.6",graphql:"^16.6.0",tslib:"^2.5.3",viem:"^1.2.0",zod:"^3.21.4"};var devDependencies={"@graphql-codegen/cli":"4.0.1","@graphql-codegen/fragment-matcher":"^3.3.1","@graphql-codegen/introspection":"^3.0.1","@graphql-codegen/typed-document-node":"^4.0.1","@graphql-codegen/typescript":"2.8.0","@graphql-codegen/typescript-operations":"^2.5.5","@pollyjs/adapter-node-http":"^6.0.5","@pollyjs/core":"^6.0.5","@pollyjs/persister-fs":"^6.0.5","@types/jest":"^29.5.1","@types/mocha":"^10.0.1","@types/node":"^18.13.0","@types/supertest":"^2.0.12",dotenv:"^16.0.3","eslint-plugin-no-only-tests":"^3.1.0",supertest:"^6.3.3"};var scripts={codegen:"npx graphql-codegen --require dotenv/config"};var exports$1={".":{"import":"./esm/src/index.js",require:"./cjs/index.js"},"./api":{"import":"./esm/src/api/index.js","default":"./cjs/src/api/index.js",types:"./esm/src/api/index.d.ts"},"./core":{"import":"./esm/src/core/index.js","default":"./cjs/src/core/index.js",types:"./esm/src/core/index.d.ts"}};var typesVersions={"*":{api:["./esm/src/api/index.d.ts"],core:["./esm/src/core/index.d.ts"]}};var packageJson = {name:name,repository:repository,license:license,version:version,main:main,module:module$1,types:types,sideEffects:sideEffects,dependencies:dependencies,devDependencies:devDependencies,scripts:scripts,exports:exports$1,typesVersions:typesVersions};

class API {
    constructor({ graphApiKey, additionalHeaders, defaultChain, } = {}) {
        if (!graphApiKey) {
            console.warn('QuickNode SDK warning: no apiKey provided. Access with no apiKey is heavily rate limited and intended for development use only. For higher rate limits or production usage, create an account on https://www.quicknode.com/');
        }
        this.graphApiKey = graphApiKey;
        this.additionalHeaders = additionalHeaders;
        this.urqlClient = this.createUrqlClient();
        this.customUrqlClient = new CustomUrqlClient(this.urqlClient);
        this.defaultChain = defaultChain || DEFAULT_CHAIN;
        this.nfts = new NftsController(this.customUrqlClient, this.defaultChain);
        this.tokens = new TokensController(this.customUrqlClient, this.defaultChain);
        this.utils = new UtilsController(this.customUrqlClient, this.defaultChain);
        this.contracts = new ContractsController(this.customUrqlClient, this.defaultChain);
        this.transactions = new TransactionsController(this.customUrqlClient, this.defaultChain);
        this.events = new EventsController(this.customUrqlClient, this.defaultChain);
        // Re-export the Urql client configured to use the Graph API for use with custom queries
        this.graphApiClient = this.urqlClient;
    }
    createUrqlClient() {
        const headers = { ...this.additionalHeaders };
        if (this.graphApiKey)
            headers['x-api-key'] = this.graphApiKey;
        headers['x-quicknode-sdk'] = 'js-sdk';
        headers['x-quicknode-sdk-version'] = packageJson?.version || 'n/a';
        const useNftKey = (data) => `${data['contractAddress']}:${data['tokenId']}`;
        const useAddressAsKey = (data) => `${data['address']}`;
        const useTransactionHashAndIndex = (data) => `${data['transactionHash']}:${data['transferIndex']}`;
        const urqlCache = exchangeGraphcache.cacheExchange({
            schema,
            keys: {
                EVMSchemaType: () => null,
                Collection: useAddressAsKey,
                CollectionOHLCVChart: () => null,
                Contract: useAddressAsKey,
                ERC721NFT: useNftKey,
                ERC721Collection: useAddressAsKey,
                ERC1155NFT: useNftKey,
                ERC1155Collection: useAddressAsKey,
                GasPrice: () => null,
                NFT: useNftKey,
                NFTContract: useAddressAsKey,
                TokenAttribute: () => null,
                TokenContract: useAddressAsKey,
                TokenEvent: useTransactionHashAndIndex,
                TokenMintEvent: useTransactionHashAndIndex,
                TokenBurnEvent: useTransactionHashAndIndex,
                TokenSaleEvent: useTransactionHashAndIndex,
                TokenSwapEvent: useTransactionHashAndIndex,
                TokenTransferEvent: useTransactionHashAndIndex,
                TokenUpload: () => null,
                OpenSeaMetadata: () => null,
                Transaction: (data) => `${data['hash']}`,
                TrendingCollection: () => null,
                Wallet: (data) => `${data['address']}`,
                WalletNFT: () => null,
                WalletTokenBalance: () => null,
            },
        });
        const client = new core.Client({
            fetch: fetch__default["default"],
            url: process.env['NX_GRAPHQL_API_URI'] ||
                'https://api.quicknode.com/graphql',
            exchanges: [urqlCache, core.fetchExchange],
            fetchOptions: () => ({ headers }),
        });
        return client;
    }
}

// We can't dynamically extend the viem client based on which add-ons are specified, which would
// make the types fully align with the add-ons for the client, so instead we do run-time checks to
// prevent calling add-ons not explicitly added in the config
function checkAddOnEnabled(enabled, humanName, configName) {
    if (!enabled) {
        throw new Error(`${humanName} is not set as enabled. Please ensure the addon is enabled on your QuickNode endpoint and enable ${configName} in the Core configuration argument`);
    }
}

const qnFetchNFTInputSchema = zod.z
    .object({
    wallet: isEvmAddress,
    contracts: zod.z.array(isEvmAddress).nullish(),
    omitFields: zod.z.array(zod.z.string()).nullish(),
})
    .merge(rpcPaginationParams)
    .strict();

const qnFetchNFTCollectionDetailsInputSchema = zod.z
    .object({
    contracts: zod.z.array(isEvmAddress),
})
    .strict();

const qnFetchNFTsByCollectionInputSchema = zod.z
    .object({
    collection: isEvmAddress,
    tokens: zod.z.array(zod.z.string()).nullish(),
    omitFields: zod.z.array(zod.z.string()).nullish(),
})
    .merge(rpcPaginationParams)
    .strict();

const qnGetTransfersByNFTInputSchema = zod.z
    .object({
    collection: isEvmAddress,
    collectionTokenId: zod.z.string(),
})
    .merge(rpcPaginationParams)
    .strict();

const qnVerifyNFTsOwnerInputSchema = zod.z
    .object({
    wallet: isEvmAddress,
    contracts: zod.z.array(zod.z.string()), // TODO: make this enforce the address:id format
})
    .strict();

const qnGetTokenMetadataByCAInputSchema = zod.z
    .object({
    contract: isEvmAddress,
})
    .strict();

const qnGetTokenMetadataBySymbolInputSchema = zod.z
    .object({
    symbol: zod.z.string(),
})
    .merge(rpcPaginationParams)
    .strict();

const qnGetTransactionsByAddressInputSchema = zod.z
    .object({
    address: isEvmAddress,
    fromBlock: zod.z.number().positive().nullish(),
    toBlock: zod.z.number().positive().nullish(),
})
    .merge(rpcPaginationParams)
    .strict()
    .refine(({ fromBlock, toBlock }) => {
    if (fromBlock && toBlock)
        return fromBlock < toBlock;
    return true;
}, { message: 'fromBlock must be less than toBlock' });

const qnGetWalletTokenBalanceInputSchema = zod.z
    .object({
    wallet: isEvmAddress,
    contracts: zod.z.array(isEvmAddress).nullish(),
})
    .merge(rpcPaginationParams)
    .strict();

const qnGetWalletTokenTransactionsInputSchema = zod.z
    .object({
    address: isEvmAddress,
    contract: isEvmAddress,
    fromBlock: zod.z.number().positive().nullish(),
    toBlock: zod.z.number().positive().nullish(),
})
    .merge(rpcPaginationParams)
    .strict();

function nftAndTokenValidator(config, schema, args) {
    checkAddOnEnabled(config.addOns?.nftTokenV2 ?? false, 'NFT And Token RPC API V2', 'nftTokenV2');
    // Uses zod to validate schema at runtime
    const validation = schema.safeParse(args);
    if (!validation.success) {
        const formattedErrors = formatErrors(validation.error);
        if (formattedErrors)
            throw formattedErrors;
    }
}
const nftAndTokenActions = (client, config) => ({
    async qn_fetchNFTCollectionDetails(args) {
        nftAndTokenValidator(config, qnFetchNFTCollectionDetailsInputSchema, args);
        const response = await client.request({
            method: 'qn_fetchNFTCollectionDetails',
            params: [args],
        });
        return response;
    },
    async qn_fetchNFTs(args) {
        nftAndTokenValidator(config, qnFetchNFTInputSchema, args);
        const response = await client.request({
            method: 'qn_fetchNFTs',
            params: [args],
        });
        return response;
    },
    async qn_fetchNFTsByCollection(args) {
        nftAndTokenValidator(config, qnFetchNFTsByCollectionInputSchema, args);
        const response = await client.request({
            method: 'qn_fetchNFTsByCollection',
            params: [args],
        });
        return response;
    },
    async qn_getTransfersByNFT(args) {
        nftAndTokenValidator(config, qnGetTransfersByNFTInputSchema, args);
        const response = await client.request({
            method: 'qn_getTransfersByNFT',
            params: [args],
        });
        return response;
    },
    async qn_verifyNFTsOwner(args) {
        nftAndTokenValidator(config, qnVerifyNFTsOwnerInputSchema, args);
        const response = await client.request({
            method: 'qn_verifyNFTsOwner',
            params: [args],
        });
        return response;
    },
    async qn_getTokenMetadataByContractAddress(args) {
        nftAndTokenValidator(config, qnGetTokenMetadataByCAInputSchema, args);
        const response = await client.request({
            method: 'qn_getTokenMetadataByContractAddress',
            params: [args],
        });
        return response;
    },
    async qn_getTokenMetadataBySymbol(args) {
        nftAndTokenValidator(config, qnGetTokenMetadataBySymbolInputSchema, args);
        const response = await client.request({
            method: 'qn_getTokenMetadataBySymbol',
            params: [args],
        });
        return response;
    },
    async qn_getTransactionsByAddress(args) {
        nftAndTokenValidator(config, qnGetTransactionsByAddressInputSchema, args);
        const response = await client.request({
            method: 'qn_getTransactionsByAddress',
            params: [args],
        });
        return response;
    },
    async qn_getWalletTokenBalance(args) {
        nftAndTokenValidator(config, qnGetWalletTokenBalanceInputSchema, args);
        const response = await client.request({
            method: 'qn_getWalletTokenBalance',
            params: [args],
        });
        return response;
    },
    async qn_getWalletTokenTransactions(args) {
        nftAndTokenValidator(config, qnGetWalletTokenTransactionsInputSchema, args);
        const response = await client.request({
            method: 'qn_getWalletTokenTransactions',
            params: [args],
        });
        return response;
    },
});

const ETH_MAINNET_NETWORK = 'ethereum-mainnet';
const qnChainToViemChain = {
    'arbitrum-mainnet': chains.arbitrum,
    'arbitrum-goerli': chains.arbitrumGoerli,
    'avalanche-mainnet': chains.avalanche,
    'avalanche-testnet': chains.avalancheFuji,
    'base-goerli': chains.baseGoerli,
    ['bsc']: chains.bsc,
    'bsc-testnet': chains.bscTestnet,
    'celo-mainnet': chains.celo,
    ['fantom']: chains.fantom,
    ['xdai']: chains.gnosis,
    ['gnosis']: chains.gnosis,
    'ethereum-goerli': chains.goerli,
    'harmony-mainnet': chains.harmonyOne,
    [ETH_MAINNET_NETWORK]: chains.mainnet,
    ['optimism']: chains.optimism,
    'optimism-goerli': chains.optimismGoerli,
    ['matic']: chains.polygon,
    ['polygon']: chains.polygon,
    'matic-testnet': chains.polygonMumbai,
    'zkevm-mainnet': chains.polygonZkEvm,
    'zkevm-testnet': chains.polygonZkEvmTestnet,
    'ethereum-sepolia': chains.sepolia,
};
function chainNameFromEndpoint(endpointUrl) {
    let hostnameParts;
    try {
        const parsedUrl = new URL(endpointUrl);
        hostnameParts = parsedUrl.hostname.split('.');
    }
    catch (e) {
        throw new QNInvalidEndpointUrl();
    }
    const quiknode = hostnameParts.at(-2);
    const chainOrDiscover = hostnameParts.at(-3);
    if (quiknode !== 'quiknode' || !chainOrDiscover)
        throw new QNInvalidEndpointUrl();
    const indexOfName = chainOrDiscover === 'discover' ? -4 : -3;
    const lengthOfEthereum = chainOrDiscover === 'discover' ? 4 : 3;
    if (hostnameParts.length === lengthOfEthereum)
        return ETH_MAINNET_NETWORK;
    const potentialChainName = hostnameParts.at(indexOfName);
    if (potentialChainName)
        return potentialChainName;
    throw new QNInvalidEndpointUrl();
}
function deriveChainFromUrl(endpointUrl) {
    const chainName = chainNameFromEndpoint(endpointUrl);
    const viemChain = qnChainToViemChain[chainName];
    if (viemChain)
        return viemChain;
    throw new QNChainNotSupported(endpointUrl);
}

function setupGlobalFetch() {
    // Required for viem to work in node
    if (!globalThis.fetch) {
        globalThis.fetch = fetch__default["default"];
        globalThis.Headers = fetch.Headers;
        globalThis.Request = fetch.Request;
        globalThis.Response = fetch.Response;
    }
}

const buildQNActions = (config) => {
    return (client) => ({
        ...nftAndTokenActions(client, config),
    });
};
class Core {
    constructor({ endpointUrl, chain, config = {} }) {
        setupGlobalFetch();
        this.endpointUrl = endpointUrl;
        const baseClient = viem.createClient({
            chain: chain || deriveChainFromUrl(endpointUrl),
            transport: viem.http(this.endpointUrl),
        }).extend(viem.publicActions);
        const qnClient = baseClient.extend(buildQNActions(config));
        this.client = qnClient;
    }
}

const QuickNode = {
    API: API,
    Core: Core,
};

Object.defineProperty(exports, 'gql', {
    enumerable: true,
    get: function () { return core.gql; }
});
exports.viem = viem__namespace;
exports.API = API;
exports.Core = Core;
exports.QNChainNotSupported = QNChainNotSupported;
exports.QNInputValidationError = QNInputValidationError;
exports.QNInvalidEndpointUrl = QNInvalidEndpointUrl;
exports["default"] = QuickNode;
